//////////////////////////////////////////////////////////////////////////////////
//
// File			: server.cpp
//
// Author		: Tom Stachowiak
//
// Date			: 3/19/98
//
// Description	: Server class for the CPS project
//
//////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <winsock.h>

#include "server.h"

////////////////////////////////////////////////////
//
// Server initialization
//
int Server::init(int prt, int len)
{
	struct sockaddr_in addr;

	port = prt;
	state = IDLE;
	log = stderr;
	size = len;
	WSADATA wsaData;	

	buffer = new char[size];

	if(WSAStartup(257, &wsaData) != 0)
	{
		fprintf(log, "Server: cannot initialize Windows sockets\n");
		return 0;
	}

	// Open main server datagram (UDP) socket
	if ((mainSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
		fprintf(log, "Server: cannot open socket\n");
		return 0;
    }
  
	// Set binding address
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	addr.sin_port        = htons(port);
  
	// Bind socket
	if(bind(mainSocket, (struct sockaddr *)&addr, sizeof(struct sockaddr)) < 0)
    {
		fprintf(log, "Server: cannot bind socket\n");
		return 0;
    }

	fprintf(log, "Server started\n");

	return 1;
}

////////////////////////////////////////////////////
//
// Server 
//
void Server::close()
{
	closesocket(mainSocket);
	delete buffer;
}

////////////////////////////////////////////////////
//
// Main server running loop 
//
void Server::run()
{
	struct timeval timeout;
	fd_set set;
	char *data;
	int n;
	int remote_addr_len;
	struct sockaddr_in remote_addr;
	
	// set timeout
	timeout.tv_sec = 0;
	timeout.tv_usec = 1000;

	// infinite reading loop
	while(TRUE)
	{
		// check on mainSocket
		FD_ZERO(&set);
		FD_SET(mainSocket, &set);
		
		// received message?
		if(select(mainSocket + 1, &set, 0, 0, &timeout) > 0)
		{
			if(FD_ISSET(mainSocket, &set))
			{
				data = new char[MAX_MSG_SIZE];
				n = recvfrom(mainSocket, data, MAX_MSG_SIZE, 0, (struct sockaddr *)&remote_addr, &remote_addr_len); 
      
				if (n < 0)
				{
					fprintf(log, "Server: recvfrom error\n");
				}
				else
					handleMessage(data, remote_addr_len, &remote_addr);

				delete data;
			}
		}
		
	}
}

////////////////////////////////////////////////////
//
// Incoming messages handler 
//
void Server::handleMessage(char *msg, int len, struct sockaddr_in *addr)
{
	fprintf(log, "Server: processing request\n");

	switch (msg[0])
	{
	case PLAY:
		fprintf(log, "Server: play request\n");		
		state = BUSY;
		process(&msg[1], len - 1, addr);
		state = IDLE;
		break;
	}
}

////////////////////////////////////////////////////
//
// Play request processing 
//
int Server::process(char *filename, int len, struct sockaddr_in *addr)
{
	int no = 0;
	
	fprintf(log, "Server: opening file %s\n", filename);

	dataFile = fopen(filename, "rb");
	if(dataFile == NULL)
	{
		fprintf(log, "Server: file opening error\n");
		return 0;
	}
	else
	{
		fprintf(log, "Server: playing\n");

		while(!feof(dataFile))
		{
			if(fread(buffer, 1, size, dataFile) == size)
			{
				sendData(addr);
				no++;
				Sleep(50);
			}
		}

		fprintf(log, "Server: end of file, packets: %d\n", no);

	}

	return 1;
}

////////////////////////////////////////////////////
//
// Data sending method 
//
int Server::sendData(struct sockaddr_in *addr)
{
	int n;

	n = sendto(mainSocket, buffer, size, 0, (struct sockaddr *)addr, sizeof(struct sockaddr_in));
	if(n < 0)
	{
		fprintf(log, "Server: sending data error\n");
		return 0;  
	}

	fprintf(log, "Server: send data\n");
	return 1;
		
}

////////////////////////////////////////////////////
//
// Main
//
void main(void)
{
	Server srv;

	if(srv.init(SERVER_PORT, 1600) == 1)
	{
		srv.run();
		srv.close();
	}
}

