//////////////////////////////////////////////////////////////////////
//
// File					: conference.cpp
//
// Description			: Conference management functions
//
// Author				: Tomasz Stachowiak, NPAC
//
// Date					: 11-13-96
//
///////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#include <mmsystem.h>

#include "NCSmanager.h"
#include "NCSmanagerDlg.h"
#include "AddUsers.h"
#include "ncs.h"
#include "conference.h"

// main dialog pointer
CNCSmanagerDlg *main_dlg;

// conference data
int conferenceExists = 0;
int iaminvited = 0;
NCSparticipant inviting_part;

//reading thread data
HANDLE hReadThread;
DWORD dwReadThreadId;

//ringing thread data
HANDLE hRingThread;
DWORD dwRingThreadId;

// plugins data
PLUGIN plugin[20];
int numPlugs;

// info about daemon process
PROCESS_INFORMATION daemon_info;

NCSparticipant activeParts[100];
int activePartsNum;

////////////////////////////////////////////////////////////////////////
//
// Conference management
//
////////////////////////////////////////////////////////////////////////
int start_conference(CNCSmanagerDlg *dlg)
{
	char path[100];
	char name[20] = "NCS_PATH";
	int len;
	main_dlg = dlg;
	 
	main_dlg->SetStatus("Initialization...please wait");

	//////////////////////////////////////////////////////////
	// version 1 -- environment variable
	//len = GetEnvironmentVariable(name, path, 80);
	//
	// version 2 -- registry key
	//tmp = AfxGetApp()->GetProfileString("", "PATH", "");
	//if(tmp.GetLength() == 0)
	//{
	//	MessageBox(NULL, "Registry not properly updated!", "Error", MB_ICONSTOP | MB_OK);
	//	exit(0);
	//}
	//strcpy(path, tmp.GetBuffer(100));
	//
	// version 3 -- application path
	getStartPath(path);
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, DAEMON_PATH);
	
	startProcess(path, &daemon_info);

	Sleep(1500);

	//////////////////////////////////
	// NCS initialization            
	if(!ncsSessionInitialize())
	{
		MessageBox(NULL, "Session initialization error", "Error", MB_OK);
		return 0;
	}

	//////////////////////////////////
	// Callbacks setting
	ncsSessionAddCallback(indLocalSessionInvited, invited);
	ncsSessionAddCallback(indLocalSessionAddMember, new_member);
	ncsSessionAddCallback(indLocalSessionDeleteMember, left_member);
	ncsSessionAddCallback(indLocalSessionStartApp, start_app);
	ncsSessionAddCallback(indServerUsers, server_users);
    ncsSessionAddCallback(indLocalSessionTimeout, timeout);
    ncsSessionAddCallback(indLocalSessionConnected, connected);
    ncsSessionAddCallback(indLocalSessionAccept, accepted);
    ncsSessionAddCallback(indLocalSessionReject, rejected);

	//////////////////////////////////
	// Reading thread creation
	hReadThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) readingThread, // thread function 
        NULL,									// argument to thread function 
        0,										// use default creation flags 
        &dwReadThreadId);						// returns the thread identifier 
	if (hReadThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	
	read_plugins();

	
	//ncsServerUsers();
	
	main_dlg->SetStatus("Conferencing panel ready to use");

	return 1;
}

void exit_conference(void)
{
	main_dlg->SetStatus("Quitting...please wait");
	TerminateThread(hReadThread, 0);
	CloseHandle(hReadThread);
	ncsSessionQuit();
	quit_plugins();
	Sleep(2000);
	TerminateProcess(daemon_info.hProcess, 0);
}


void conf_reset(void)
{
	main_dlg->SetStatus("Conference reset...please wait");
	exit_conference();
	main_dlg->DeleteAllMembers();
	main_dlg->DeleteAllApps();
	conferenceExists = 0;
	Sleep(1000);
	start_conference(main_dlg);
}

void accept(void)
{
	if(iaminvited)
	{
		iaminvited = 0;
		if(!conferenceExists)
		{
			conferenceExists = 1;
		}
		// main_dlg->MessageBox(inviting_part.host);
		ncsSessionAcceptInvitation(&inviting_part);

		stopRing();
	}
}

void reject(void)
{
	if(iaminvited)
	{
		iaminvited = 0;
		ncsSessionRejectInvitation(&inviting_part);

		stopRing();
	}
}


////////////////////////////////////////////////////////////////////////
//
// Members management
//
////////////////////////////////////////////////////////////////////////
void parseIPaddress(char *IP, char *name, char *host)
{
  char *ptr, *pos;
  int len, i;

  ptr = IP;
  pos = strchr(ptr, 64); /* code of @ */
  if(pos != NULL)
  {
	len = (int)pos - (int)ptr;
	memcpy(name, ptr, len);
	name[len]='\0';
	ptr += len+1;
	memcpy(host, ptr, strlen(IP)-len-1);
	host[strlen(IP)-len-1]='\0';
  }
  else
  {
	  strcpy(name, ptr);
	  host = NULL;
  }
} /* parseIPaddress() */

void add_member(char *user, char *host)
{
	ncsSessionAddMember(user, host);
}

void add_user(char *member)
{
	NCSparticipant part;

	if(!conferenceExists)
	{
		ncsSessionCreate();
		conferenceExists = 1;
	}
	parseIPaddress(member, part.user, part.host);		
	
	if(part.host != NULL)
		ncsSessionAddMember(part.user, part.host);
}

void confAddMemberDlg()
{
	CAddUsers dlg;
	char text[100];

	ncsServerUsers();

	int nResponse = dlg.DoModal();

	if (nResponse == IDOK)
	{
	}	
	else if (nResponse == IDCANCEL)
	{
	}

}

////////////////////////////////////////////////////////////////////////
//
// Archive 
//
////////////////////////////////////////////////////////////////////////
void confArchivePlay()
{
	ncsSessionRestoreInit();
}

void confArchivePlayStop()
{
	ncsSessionRestoreClose();
}

void confArchiveRec()
{
	ncsSessionSaveInit();
}

void confArchiveRecStop()
{
	ncsSessionSaveClose();
}

////////////////////////////////////////////////////////////////////////
//
// Applications management
//
////////////////////////////////////////////////////////////////////////
void read_plugins(void)
{
	FILE *file;
	int i, len;
	char path[100];
	char plugin_path[100];

	getStartPath(path);
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, PLUGINS_CFG_PATH);

	if((file = fopen(path, "r")) == NULL)
		MessageBox(NULL, "Can't read file", "Error", MB_OK);
	else
	{
		fscanf(file, "%d\n", &numPlugs);
		for(i=0; i<numPlugs; i++)
		{
			fscanf(file, "%d\n", &plugin[i].port);
			fgets(plugin[i].name, 50, file);
			plugin[i].name[strlen(plugin[i].name)-1] = 0;
			getStartPath(plugin[i].path);
			fgets(plugin_path, 100, file);
			strcat(plugin[i].path, plugin_path);
			plugin[i].path[strlen(plugin[i].path)-1] = 0;
			plugin[i].started = 0;
			main_dlg->AddApp(plugin[i].name);
		}
	}
}


void quit_plugins(void)
{
	int i;
	
	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].started)
		{
			ncsSessionSendToApp(plugin[i].port, indLocalAppQuit, NULL, 0);
			plugin[i].started = 0;
		}
	//		TerminateProcess(plugin[i].info.hProcess, 0);
	}
}

void startProcess(char *path, PROCESS_INFORMATION *pinfo)
{
	STARTUPINFO sinfo;

	sinfo.cb = sizeof(STARTUPINFO);
	sinfo.lpReserved = NULL;
	sinfo.lpDesktop = NULL;
	sinfo.lpTitle = NULL;
	sinfo.dwFlags = STARTF_FORCEONFEEDBACK;
	sinfo.cbReserved2 = 0;
	sinfo.lpReserved2 = NULL;

	if(CreateProcess(path, NULL, NULL, NULL, FALSE, 
					NORMAL_PRIORITY_CLASS | DETACHED_PROCESS, NULL, NULL, 
					&sinfo, pinfo)
		== FALSE)
			MessageBox(NULL, "Create process error", "Error", MB_OK);
}


int app_started(int index)
{
	return plugin[index].started;
}

void create_app(int port)
{
	int found = 0, index, i;

	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].port == port)
		{
			index = i;
			found = 1;
		}
	}
	if((found) && (!plugin[index].started))
	{
		// MessageBox(NULL, plugin[index].path, "Info", MB_OK);
		ncsSessionStartApp(port);
		plugin[index].started = 1;
		startProcess(plugin[index].path, &plugin[index].info);

	}
}

void start_local_app(int index)
{
	create_app(plugin[index].port);
}

void quit_application(int port)
{
	int found = 0, index, i;

	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].port == port)
		{
			index = i;
			found = 1;
		}
	}
	if((found) && (plugin[index].started))
	{
		plugin[index].started = 0;
		ncsSessionSendToApp(port, indLocalAppQuit, NULL, 0);
		//		TerminateProcess(plugin[index].info.hProcess, 0);
	}
}

void quit_local_app(int index)
{
	if(plugin[index].started)
	{
		plugin[index].started = 0;
		ncsSessionSendToApp(plugin[index].port, indLocalAppQuit, NULL, 0);
		//TerminateProcess(plugin[index].info.hProcess, 0);
	}
}

void info_app(int index)
{
	char text[200];

	wsprintf(text, "Port: %d\nName: %s\nPath: %s\n", plugin[index].port,
													 plugin[index].name,
													 plugin[index].path);
	main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONINFORMATION);
}

/////////////////////////////////////////////////////////////////////////
//
// NCS callbacks
//
/////////////////////////////////////////////////////////////////////////
void new_member(void *data)
{
	NCSparticipant *part;

	part = (NCSparticipant *)data;
	main_dlg->SetStatus("New participant joins");
	main_dlg->AddMember(part->user, part->host);
}

void left_member(void *data)
{
	char text[200];
	NCSparticipant *part;

	part = (NCSparticipant *)data;
	main_dlg->DeleteMember(part->user, part->host);
	wsprintf(text, "%s@%s left", part->user, part->host);
	main_dlg->SetStatus(text);
	//main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
}

void invited(void *data)
{
	char text[200];
	NCSparticipant *part;

	part = (NCSparticipant *)data;
	memcpy(&inviting_part, part, sizeof(NCSparticipant));
	iaminvited = 1;
	wsprintf(text, "RECEIVED INVITATION from %s@%s", 
			 part->user, part->host);
	main_dlg->SetStatus(text);
	main_dlg->Invitation();
	//main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
	
	startRing();
}

void start_app(void *data)
{
	int *port;

	port = (int *)data;
	create_app(*port);
}

void server_users(void *data)
{
	InviteData *users;
	char text[100];
	int i;

	users = (InviteData *)data;
	activePartsNum = ntohl(users->numMembers);
	for(i=0; i<activePartsNum; i++)
	{
		memcpy(&activeParts[i], &users->parts[i], sizeof(NCSparticipant));
	}
}

void timeout(void *data)
{
	NCSparticipant *part;
	char text[100];

	part = (NCSparticipant *)data;
	wsprintf(text, "%s@%s not present", part->user, part->host);
	main_dlg->SetStatus(text);
	//main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
}

void connected(void *data)
{
	NCSparticipant *part;
	char text[100];

	part = (NCSparticipant *)data;
	wsprintf(text, "%s@%s received invitation", part->user, part->host);
	main_dlg->SetStatus(text);
	//main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
}

void accepted(void *data)
{
	NCSparticipant *part;
	char text[100];

	part = (NCSparticipant *)data;
	wsprintf(text, "%s@%s accepted invitation", part->user, part->host);
	main_dlg->SetStatus(text);
	//main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
}

void rejected(void *data)
{
	NCSparticipant *part;
	char text[100];

	part = (NCSparticipant *)data;
	wsprintf(text, "%s@%s rejected invitation", part->user, part->host);
	main_dlg->SetStatus(text);
	//main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
}

/////////////////////////////////////////////////////////////////////////
//
// Reading thread
//
/////////////////////////////////////////////////////////////////////////
DWORD readingThread(LPDWORD data)
{
	struct timeval timeout;
	fd_set set;
	int sockfd;

	timeout.tv_sec = 0;
	timeout.tv_usec = 500000;
	while(1)
	{
		sockfd = ncsSessionGetFD(&set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsProcessIncoming(&set);
		}
	}

	return 1;
}

/////////////////////////////////////////////////////////////////////////
//
// Ringing
//
/////////////////////////////////////////////////////////////////////////
void startRing()
{
	//////////////////////////////////
	// Ringing thread creation
	hRingThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) ringingThread, // thread function 
        NULL,									// argument to thread function 
        0,										// use default creation flags 
        &dwRingThreadId);						// returns the thread identifier 
	if (hRingThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
}

void stopRing()
{
	TerminateThread(hRingThread, 0);
	main_dlg->resetRing();
}

DWORD ringingThread(LPDWORD data)
{
	int len, i;
	CString tmp;
	char path[100];
	char name[20] = "NCS_PATH";

	getStartPath(path);
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, "RINGIN.WAV");
	
	for(i=0; i<10; i++)
	{
		main_dlg->setRing();
		PlaySound(path, NULL, SND_SYNC | SND_NODEFAULT);
		main_dlg->resetRing();
		Sleep(2000);
	}

	return 1;
}


void getStartPath(char *path)
{
	int i;
	char *tmp;
	int found = 0;
	char p[100];

	tmp = GetCommandLine();
	strcpy(p, tmp);
	i = strlen(p) - 1;
	while(!found && (i >= 0))
	{
		if(p[i] == '\\')
		{
			p[i] = 0;
			found = 1;
		}
		i--;
	}
	if(!found)
		strcpy(p, "");
	
	if(p[0] == '"')
		strcpy(path, &p[1]);
	else
		strcpy(path, p);

}

////////////////////////////////  END  /////////////////////////////////////