/*********************************************************************
 *
 * File:         message_send.c
 *
 * Description:  Messages send module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         02-06-97
 *
 *********************************************************************/
#include <stdio.h>

#include "global.h"
#include "session_manage.h"
#include "daemon.h"

extern char localHost[];   /* local host name */

extern int tmpSocket;
extern int localSocket;
extern int inviteSocket;
extern int mbrSocket[];
extern int appSocket[];

int getHostFullName(char *name, int len)
{
  char tmp[100];
  struct hostent *hostinfo;
  
  gethostname(tmp, 100);
  
  hostinfo = gethostbyname(tmp);
  if(hostinfo == NULL)
    {
      return 0;
    }
  
  strcpy(name, hostinfo->h_name);
  
  return 1;
}

int gethostid()
{
  char tmp[100];
  struct hostent *hostinfo;
  
  gethostname(tmp, 100);
  
  hostinfo = gethostbyname(tmp);
  if(hostinfo == NULL)
    {
      return -1;
    }

  return *(int *)hostinfo->h_addr;
}


int getHostName(int id, char *host)
{
  struct hostent *hostinfo;
  
  hostinfo = gethostbyaddr((const char *)&id, sizeof(int), AF_INET);
  if(hostinfo == NULL)
    {
      return 0;
    }
  
  strcpy(host, hostinfo->h_name);
  return 1;
}


int writeAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = send( fd, (char *)buffer, len, 0 );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "writeAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}

int sendControlMessage(int sock, MsgHeader *header, char *msg, int len)
{
  char buf[MSG_HEADER_LENGTH];
  int length;

  header->session = getSessionID();

  buf[0] = header->tag;
  buf[1] = header->session;
  buf[2] = header->sender.handle;
  strcpy(&buf[3], header->sender.user);
  //header->sender.host = htonl(header->sender.host);
  memcpy(&buf[23], &header->sender.host, sizeof(int));
  buf[27] = header->receiver.handle;
  strcpy(&buf[28], header->receiver.user);
  //header->receiver.host = htonl(header->receiver.host);
  memcpy(&buf[48], &header->receiver.host, sizeof(int));
  length = htonl(len);
  memcpy(&buf[52], &length, sizeof(int));

  if(sock >= 0)
    {
      if(writeAll(sock, buf, MSG_HEADER_LENGTH) < 0)
	{
	  fprintf(stderr, "Engine: write error\n");
	  return 0;
	}
      if(writeAll(sock, msg, len) < 0)
	{
	  fprintf(stderr, "Engine: write error\n");
	  return 0;
	}
    }
  else
    return 0;

  return 1;
}


int sendLocalMessage(short tag, char *msg, int len)
{
  MsgHeader header;
  DWORD user_len = 20;

  header.tag = (unsigned char)tag;
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.dataLength = len;
  sendControlMessage(localSocket, &header, msg, len);

  return 1;
}


int sendMemberMessage(int no, short tag, char *msg, int len)
{
  MsgHeader header;
  int sent = 0;
  int i;
  DWORD user_len = 20;

  PRINTF("Engine: Send message to member %d\n", no);
  header.tag = (unsigned char)tag;
  header.sender.handle = getLocalHandle();
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.dataLength = len;

  if(no > 0)        /* send to particular member */
    {
      if(getReceiverData(no, &header))
	{
	  PRINTF("User %d, socket %d\n", no, mbrSocket[no]); 
	  sendControlMessage(mbrSocket[no], &header, msg, len);
	}
    }

  if(no == 0)       /* send to statekeeper */
    {
      i = 0;
      while((i < MAX_MEMBERS) && (!sent))
	{
	  if(getReceiverData(i, &header))
	    {
	      PRINTF("User %d, socket %d\n", i, mbrSocket[i]); 
	      if(sendControlMessage(mbrSocket[i], &header, msg, len) == 0)
		  return 0;
	      sent = 1;
	    }
	  i++;
	}
    }
  if(no < 0)        /* send to all members */
    {
      for(i=0; i<MAX_MEMBERS; i++)
	{
	  if(getReceiverData(i, &header))
	    {
	      PRINTF("User %d, socket %d\n", i, mbrSocket[i]); 
	      if(sendControlMessage(mbrSocket[i], &header, msg, len) == 0)
		  return 0;
	    }
	}
    }

  return 1;
}


int connect2CE(int hostid)
{
  struct sockaddr_in addr;
  int sock;
  char host[50];

  if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
     {
       fprintf(stderr, "Engine: socket opening error\n");
       return 0;
     }

  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = hostid; 
  addr.sin_port = htons(DAEMON_PORT);

  // getHostName(hostid, host);
  // PRINTF("Connecting to host %s, id %d...\n", host, hostid);
  if(connect(sock, (struct sockaddr *)&addr, sizeof(struct sockaddr)) < 0)
    {
      closesocket(sock);
      return -1;
    }

  PRINTF("Connected with socket %d\n", sock);
  return sock;
}


int sendNewMessage(Participant *part, short tag, 
			     char *msg, int len)
{
  int no;
  MsgHeader header;
  DWORD user_len = 20;

  no = getFreeMbrSocket();
  mbrSocket[no] = connect2CE(part->host);
  PRINTF("Engine: send new message to member %d, socket %d\n", no, mbrSocket[no]);
  if(mbrSocket[no] < 0)
    {
      PRINTF("Engine: cannot connect to remote engine\n");
      return 0;
    }

  header.tag = (unsigned char)tag;
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.dataLength = len;

  memcpy(&header.receiver, part, sizeof(Participant));

  sendControlMessage(mbrSocket[no], &header, msg, len);

  return 1;
}

int connect2member(int hostid)
{
  MsgHeader header;
  int sock;
  DWORD user_len = 20;

  header.tag = reqNewRemoteSessionConnect;
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.sender.handle = getLocalHandle();
  header.dataLength = 0;

  sock = connect2CE(hostid);

  if(sock >= 0)
    {
      sendControlMessage(sock, &header, NULL, 0);
    }

  return sock;
}

int sendAppMessage(int no, short tag, char *msg, int len)
{
  int i;
  MsgHeader header;
  DWORD user_len = 20;

  header.tag = (unsigned char)tag;
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.dataLength = len;
  PRINTF("Send app %d msg tag %d\n", no, tag);

  if(no >= 0)        /* send to particular member */
    {
      if(appSocket[no] > 0)
	 sendControlMessage(appSocket[no], &header, msg, len);
    }
  else
    {
      for(i=0; i<MAX_APPS; i++)
	{
	  if(appSocket[i] >= 0)
	    sendControlMessage(appSocket[i], &header, msg, len);
	}
    }

  return 1;
}

int sendInviteMessage(short tag, char *msg, int len)
{
  MsgHeader header;
  DWORD user_len = 20;

  header.tag = (unsigned char)tag;
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.dataLength = len;

  if(inviteSocket > 0)
    sendControlMessage(inviteSocket, &header, msg, len);

  return 1;
}


void getStartPath(char *path)
{
	int i;
	char *tmp;
	int found = 0;
	char p[100];

	tmp = GetCommandLine();
	strcpy(p, tmp);
	i = strlen(p) - 1;
	while(!found && (i >= 0))
	{
		if(p[i] == '\\')
		{
			p[i] = 0;
			found = 1;
		}
		i--;
	}
	if(!found)
		strcpy(p, "");
	
	if(p[0] == '"')
		strcpy(path, &p[1]);
	else
		strcpy(path, p);

}






