#include <MMSYSTEM.H>
#include <winbase.h>

#include "stdafx.h"
#include "Conference.h"
#include "TSaudioProc.h"

/////////////////////////////////
// module variables
/////////////////////////////////

#define TRESHOLD 2000

////////////////////////////////////////////////////////////////
// common audio data
WAVEFORMATEX *format;	// format of data
MMRESULT err;			// audio error code
	
////////////////////////////////////////////////////////////////
// audio reading data

// handle for in port
HWAVEIN haudio_in = NULL;	

// reading Thread 
HANDLE hThread; 
DWORD dwThreadId;

// event for synchronize reading
HANDLE hEventIn;

// wave data header
WAVEHDR waveInHeader;	

// reading state
int audioReadActive;
int first = 1;

// conference
Conference *conference;

/////////////////////////////////////////////////////////////////
// audio writing data

AudioOutPort outport;

int audioInit(Conference *conf)
{
	char in_event_name[20] = "read";
	char out_event_name[20] = "write";
	char buf_event_name[20] = "buffer";
	int i;

	conference = conf;
	
	format = (WAVEFORMATEX *)malloc(sizeof(WAVEFORMATEX));

	format->wFormatTag = WAVE_FORMAT_PCM;
	format->nChannels = 1;
	format->nSamplesPerSec = 8000;
    format->nAvgBytesPerSec = 16000; 
    format->nBlockAlign = 2; 
	format->wBitsPerSample = 16;
	format->cbSize = NULL;

	waveInHeader.lpData = new char[PORTION_SIZE];
	waveInHeader.dwBufferLength = PORTION_SIZE;
	waveInHeader.dwFlags = 0L;
	waveInHeader.dwLoops = 0L;

	audioReadActive = 0;

	// create read event
	hEventIn = CreateEvent(NULL, TRUE, TRUE, in_event_name);
	ResetEvent(hEventIn);


	outport.header[0].lpData = new char[PORTION_SIZE];
	outport.header[0].dwBufferLength = PORTION_SIZE;
	outport.header[1].lpData = new char[PORTION_SIZE];
	outport.header[1].dwBufferLength = PORTION_SIZE;

	// create write event
	outport.hEvent = CreateEvent(NULL, TRUE, TRUE, out_event_name);
	outport.hBufEvent = CreateEvent(NULL, TRUE, TRUE, buf_event_name);

	outport.active = 0;

	return 1;
}

void audioClose(void)
{
	free(format);
	CloseHandle(hEventIn);
	CloseHandle(outport.hBufEvent);
	CloseHandle(outport.hEvent);
}


//////////////////////////////////////////////////////////////////
//
// Audio reading functions
//
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// audio reading init
int audioReadInit(void)
{

	audioReadActive = 1;
	first = 1;

    // create reading thread
    hThread = CreateThread( 
        NULL,                        // no security attributes 
        0,                           // use default stack size  
        (LPTHREAD_START_ROUTINE) audioReadThread, // thread function 
        NULL,                // argument to thread function 
        0,                           // use default creation flags 
        &dwThreadId);                // returns the thread identifier 
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
		return 0;
	}

	// opening audio in port
	err = waveInOpen(&haudio_in, WAVE_MAPPER, format, (DWORD)hEventIn, NULL, 
		CALLBACK_EVENT);
	if(err != MMSYSERR_NOERROR)
	{
		MessageBox(NULL, "Can't open audio in port", "Error", MB_OK);
		return 0;
	}
	audioReadChunk();

	return 1;
}

///////////////////////////////////////////////////////////////////////////
// main audio reading thread
DWORD audioReadThread(LPDWORD lpdwParam)
{
	MSG msg;
	char text[100];

	while(1)
	{
		WaitForSingleObject(hEventIn, INFINITE);
		if(audioReadActive)
		{
			if(waveInHeader.dwFlags & WHDR_DONE)
			{
				audioReadChunk();
			}
		}
		else
		{
			MessageBox(NULL, "Thread: audio read close", "Info", MB_OK);
		}

		ResetEvent(hEventIn);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////
// audio reading close function
void audioReadClose(void)
{
	char text[100];
	
	audioReadActive = 0;
	
	TerminateThread(hThread, 0);
	CloseHandle(hThread);
	
	waveInStop(haudio_in);
	err = waveInReset(haudio_in);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't reset audio in port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	waveInUnprepareHeader(haudio_in, &waveInHeader, sizeof(WAVEHDR));
	err = waveInClose(haudio_in);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't close audio in port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	// CloseHandle(haudio_in);
	haudio_in = NULL;
}

///////////////////////////////////////////////////////////////////////////
// Reads audio portions from microphone and saves it to file
void audioReadChunk(void)
{
	DWORD size;
	void *buffer;

	if(!first)
	{	
		waveInUnprepareHeader(haudio_in, &waveInHeader, sizeof(WAVEHDR));
		buffer = malloc(PORTION_SIZE);
		memcpy(buffer, waveInHeader.lpData, PORTION_SIZE);
	}

	if(haudio_in != NULL)
	{
		waveInHeader.dwFlags = 0L;
		waveInHeader.dwLoops = 0L;

		err = waveInPrepareHeader(haudio_in, &waveInHeader, sizeof(WAVEHDR));
		if(err != MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't prepare audio header", "Error", MB_OK);
	
		ResetEvent(hEventIn);

		if(waveInAddBuffer(haudio_in, &waveInHeader, sizeof(WAVEHDR))
				!= MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't add read audio buffer", "Error", MB_OK);

		if(first)
		{
			first = 0;
			waveInStart(haudio_in);
		}
		else
			audioReadHandler(buffer);
	}
}

///////////////////////////////////////////////////////////////////////////
// Handler for read audio chunk
void audioReadHandler(void *buffer)
{
	short max = 0;
	int i;
	
	if(audioReadActive)
	{
		for(i=0; i<PORTION_SAMPLES; i++)
			if(((short *)buffer)[i] > max)
				max = ((short *)buffer)[i];
		conference->SendData((char *)buffer, max);
	}
	
	free(buffer);
}
 
///////////////////////////////////////////////////////////////////////////
// starting handling of read audio buffers
void audioReadStart(void)
{
	audioReadActive = 1;
}

///////////////////////////////////////////////////////////////////////////
// stop handling of read audio buffers
void audioReadStop(void)
{
	audioReadActive = 0;
}


//////////////////////////////////////////////////////////////////
//
// Audio writing functions
//
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// audio writing init
int audioWriteInit(void)
{
	char text[100];
	
	outport.first = 1;
	outport.handle = NULL;
	outport.headerNo = 0;
	outport.active = 1;


	if(outport.handle == NULL)
	{
		err = waveOutOpen(&outport.handle, WAVE_MAPPER, format, (DWORD)outport.hEvent, NULL, 
			CALLBACK_EVENT);
		if(err != MMSYSERR_NOERROR)
		{
			wsprintf(text, "Can't open audio out port %d", err);
			MessageBox(NULL, text, "Error", MB_OK);
			return 0;
		}	

	}
	
    // create reading thread
    outport.hThread = CreateThread( 
        NULL,                        // no security attributes 
        0,                           // use default stack size  
        (LPTHREAD_START_ROUTINE) audioWriteThread, // thread function 
        NULL,                // argument to thread function 
        0,                           // use default creation flags 
        &outport.dwWriteThreadId);                // returns the thread identifier 
	if (outport.hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
		return 0;
	}

	memset(outport.buffer, 0, PORTION_SIZE);
	audioWriteChunk();
	audioWriteChunk();

	return 1;
}

///////////////////////////////////////////////////////////////////////////
// audio writnig close
void audioWriteClose(void)
{
	char text[100];
	
	TerminateThread(outport.hThread, 0);
	CloseHandle(outport.hThread);

	err = waveOutReset(outport.handle);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't reset audio out port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	waveOutUnprepareHeader(outport.handle, &outport.header[0], sizeof(WAVEHDR));
	waveOutUnprepareHeader(outport.handle, &outport.header[1], sizeof(WAVEHDR));
	err = waveOutClose(outport.handle);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't close audio out port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	outport.handle = NULL;
}


///////////////////////////////////////////////////////////////////////////
// main audio writing thread
DWORD audioWriteThread(LPDWORD lpdwParam)
{
	MSG msg;
	char text[100];


	while(1)
	{
		WaitForSingleObject(outport.hEvent, INFINITE);
		if((outport.header[(outport.headerNo + 1) % 2].dwFlags & WHDR_DONE))
		{
			// MessageBox(NULL, "Write thread", "Error", MB_OK);
			audioWriteChunk();
		}
  }

	return 0;
}

///////////////////////////////////////////////////////////////////////////
// setting audio volume
void audioWriteVolume(UINT vol)
{
	waveOutSetVolume(outport.handle, vol);
}

///////////////////////////////////////////////////////////////////////////
// Puts audio portion to speaker
void audioWriteChunk(void)
{
	static int gap = 0;
	int nowait = 0;
	char text[100];
	
	if(!outport.first)
	{
		/*
		gap++;
		if(gap == 20)
		{
			waveOutReset(outport.handle);
			gap = 0;
		}
		*/

		ResetEvent(outport.hEvent);

		if(waveOutWrite(outport.handle, &(outport.header[outport.headerNo]), sizeof(WAVEHDR))
			!= MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't write audio", "Error", MB_OK);
		
		outport.headerNo = (outport.headerNo + 1) % 2;

	//	if(!(outport.header[outport.headerNo].dwFlags & WHDR_DONE))
	//		WaitForSingleObject(outport.hEvent, INFINITE);
		waveOutUnprepareHeader(outport.handle, &(outport.header[outport.headerNo]), sizeof(WAVEHDR));
	}
	else
		outport.first = 0;

	outport.header[outport.headerNo].dwFlags = 0L;
	outport.header[outport.headerNo].dwLoops = 0L;

	WaitForSingleObject(outport.hBufEvent, INFINITE);
	ResetEvent(outport.hBufEvent);
	memcpy(outport.header[outport.headerNo].lpData, outport.buffer, PORTION_SIZE);
	memset(outport.buffer, 0, PORTION_SIZE);
	SetEvent(outport.hBufEvent);

	err = waveOutPrepareHeader(outport.handle, &(outport.header[outport.headerNo]), sizeof(WAVEHDR));
	if(err != MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't prepare audio out header", "Error", MB_OK);
}


short audioWriteAddToBuf(void *buffer)
{
	int i;
	short max = 0;
	
	WaitForSingleObject(outport.hBufEvent, INFINITE);
	//MessageBox(NULL, "Add to buf", "Error", MB_OK);
	ResetEvent(outport.hBufEvent);
	for(i=0; i<PORTION_SAMPLES; i++)
	{
		outport.buffer[i] += ((short *)buffer)[i];
		if(outport.buffer[i] > max)
			max = outport.buffer[i];
	}
	if(max < TRESHOLD)
	{
		memset(outport.buffer, 0, PORTION_SIZE);
		max = 0;
	}
	SetEvent(outport.hBufEvent);

	return max;
}