///////////////////////////////////////////////////////////////////////////
// 
// File						: conference.cpp
//
// Description				: conference functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-21-96
//
////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NCSvideo.h"
#include "NCSvideoDlg.h"
#include "PanelDlg.h"
#include "ConnDlg.h"
#include "display.h"
#include "conference.h"
#include "ncs.h"
#include "VideoProtocol.h"
#include "h261.h"
#include "h263.h"
#include "capture.h"
#include "convert.h"

CNCSvideoDlg *main_dlg;
ncsApplication *ncsApp;
DWORD localH;
HANDLE hThread;
DWORD dwThreadId;
int state;
int code;
int intra;
BOOL H261IntraMode;
DWORD codeTime=500;
DWORD decodeTime;
HANDLE hCodecEvent;
HANDLE hCtrlThread;
DWORD dwCtrlThreadId;
BOOL test;

BOOL first[MAX_MEMBERS];
int frameNum = 0;
int lastMsgNum[MAX_MEMBERS];
int intraFrame = 0;

char connection[20];
int limit = 4;

void confInit(CNCSvideoDlg *dlg)
{
	DWORD wParam;
	int i;
	
	main_dlg = dlg;
	
	// Start up NCS
	ncsApp = (ncsApplication *)ncsAppAttach(VIDEO_PLUGIN_PORT);
	if(ncsApp == NULL)
	{
		confExit();
		return;
	}

	ncsAppAddCallback(ncsApp, tagAppStarting, (void (*)())confStarting);
	ncsAppAddCallback(ncsApp, tagAppQuit, (void (*)())confExit);

	//localH = ncsAppGetLocalHandle(ncsApp);
	
	// Create reading thread
	hThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) confReadThread,	// thread function 
			NULL,									// argument to thread function 
			0,										// use default creation flags 
			&dwThreadId);							// returns the thread identifier 
	
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

	hCtrlThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) confTimeCtrlThread,	// thread function 
			NULL,									// argument to thread function 
			0,										// use default creation flags 
			&dwCtrlThreadId);							// returns the thread identifier 
	
	if (hCtrlThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}
	
	p64InitCodec();
	p64InitEncoder();
	p64InitDecoder();

	H263initEncoder(176, 144);
	H263initDecoder();

	displayInit();

	hCodecEvent = CreateEvent(NULL, TRUE, TRUE, "codec_event");

	state = STOP;
	code = H263;
	intra = 1;
	H261IntraMode = FALSE;
	test = TRUE;

	for(i=0; i<MAX_MEMBERS; i++)
		first[i] = TRUE;

	if(!confRestoreConfig())
		strcpy(connection, "Modem 28.8");
	setConnection(connection);
}

void confStarting()
{
	confSendMessage(videoTagNewMember, NULL, 0);
}

DWORD confReadThread(LPDWORD lpdwParam)
{
	struct timeval timeout;
	fd_set set;
	ncsData *ncsdata;
	int size;
	int sockfd;
	char *data;

	timeout.tv_sec = 0;
	timeout.tv_usec = 100000;

	while(1)
	{
		sockfd = ncsAppGetFD(ncsApp, &set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsAppProcessIncoming(ncsApp, &set);
			size = ncsAppGetData(ncsApp, &set, &data);
			if(size > 0)
			{
				size = size - sizeof(ncsData) + 1;
				ncsdata = (ncsData *)data;
				ncsdata->handle = ntohl(ncsdata->handle);
				ncsdata->type = ntohl(ncsdata->type);
				confHandleData(ncsdata->type, ncsdata->data, size, 
								       ncsdata->handle);
				SetFocus(main_dlg->m_hWnd);
				free(data);
			}
		}
	}
	return 1;
}

void confHandleData(DWORD tag, void *data, int len, DWORD handle)
{
	char buffer[1600];
	VideoPluginFrame *frame;
	DWORD wParam;
	int size;
	
	// MessageBox(NULL, "Get audio data", "Info", MB_OK);
	switch (tag)
	{
	case H261:
		frame = (VideoPluginFrame *)data;		
		if((frame->num < lastMsgNum[handle]) && ((lastMsgNum[handle] - frame->num) < 100))
		{
			// misplaced packet
		}
		else
		{
			lastMsgNum[handle] = frame->num;
			size = len - sizeof(VideoPluginFrame) + 1;
			displayH261(frame->data, size, handle);
		}
		break;
    
	case H263:
		frame = (VideoPluginFrame *)data;
		if((frame->num < lastMsgNum[handle]) && ((lastMsgNum[handle] - frame->num) < 100))
		{
			// misplaced packet
		}
		else
		{
			lastMsgNum[handle] = frame->num;
			size = len - sizeof(VideoPluginFrame) + 1;
			displayH263(frame->data, size, handle);
		}
		break;
	
	case YUV9:
		frame = (VideoPluginFrame *)data;
		if((frame->num < lastMsgNum[handle]) && ((lastMsgNum[handle] - frame->num) < 100))
		{
			// misplaced packet
		}
		else
		{
			lastMsgNum[handle] = frame->num;
			size = len - sizeof(VideoPluginFrame) + 1;
			displayYUV9((unsigned char *)frame->data, size, handle);
		}
		break;
	
	case videoTagExit:
        // exit plugin 
		break;

    case videoTagConfirm:
		break;

    case videoTagPlay:
		lastMsgNum[handle] = 0;
		confSendMessage(videoTagConfirm, NULL, 0);
		if(first[handle])
		{
			//displayOpen(handle);
			wParam = CREATE_DISPLAY*65536 + handle;
			SendMessage(main_dlg->m_hWnd, WM_COMMAND, wParam, NULL); 
			first[handle] = FALSE;
		}
		break;

    case videoTagNewMember:
        if(state == PLAY)
		{
			intraFrame = 0;
	        ncsAppSendData(ncsApp, handle, videoTagPlay, NULL, 0);
		}
		break;

    case videoTagStop:
		wParam = DESTROY_DISPLAY*65536 + handle;
		SendMessage(main_dlg->m_hWnd, WM_COMMAND, wParam, NULL); 
		first[handle] = TRUE;
		break;

    case videoTagFrameDecoded:
		break;

    default:
		break;
	}
}

void confSendMessage(DWORD tag, void *data, int len)
{
	ncsAppSendData(ncsApp, -1, tag, data, len);
}


void confSendVideo(int tag, char *buf, int len)
{
	VideoPluginFrame *frame;
	int size;

	size = sizeof(VideoPluginFrame) + len - 1;
	frame = (VideoPluginFrame *)GlobalAlloc(GPTR, size);
	//gettimeofday(&frame->time);
	frameNum = (frameNum + 1) % 256;
	frame->num = (unsigned char)frameNum;
	memcpy(frame->data, buf, len);
	confSendMessage(tag, frame, size);
	GlobalFree(frame);
}

void confCodeImage(char *yuv9, int conv)
{
	char *codedBuf;
	int size;
	DWORD start, stop;
	unsigned char *yuv;
	char *yuv9qcif;
	char *buf;


	start = GetTickCount();
	
	yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_YUV_SIZE);
	codedBuf = (char *)GlobalAlloc(GPTR, 20000);

	switch(code)
	{
	case H261:
		if(conv)
		{
			YVU9toYUV411((unsigned char *)yuv9, 
					yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
					240, 180);
			buf = (char *)yuv;
		}
		else
			buf = yuv9;

		displayYUV((unsigned char *)buf, QCIF_YUV_SIZE, MAX_USERS);

		WaitForSingleObject(hCodecEvent, INFINITE);
		ResetEvent(hCodecEvent);
	
		if(!intraFrame)
		{
			size = p64EncodeIntraFrame((char *)buf, codedBuf);
		}
		else
			size = p64EncodeFrame((char *)buf, codedBuf);
	
		SetEvent(hCodecEvent);
	
		confSendVideo(H261, codedBuf, size);

		break;
	
	case H263:
		if(conv)
		{
			YVU9toYUV411((unsigned char *)yuv9, 
					yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
					240, 180);
			buf = (char *)yuv;
		}
		else
			buf = yuv9;

		displayYUV((unsigned char *)buf, QCIF_YUV_SIZE, MAX_USERS);

		if(!intraFrame)
		{
			size = H263codeIntraFrame((unsigned char *)buf, codedBuf);
			intra = 0;
		}
		else
			size = H263codeFrame((unsigned char *)buf, codedBuf);

		confSendVideo(H263, codedBuf, size);
	
		break;

	case YUV9:
		if(!conv)
		{
			code = H261;
			H261IntraMode = TRUE;
			p64setIntraMode();
		}
		else
		{
			yuv9qcif = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*9/8);
			resizeYUV9((char *)yuv9, 240, 180, (char *)yuv9qcif, 176, 144);
			displayYUV9((unsigned char *)yuv9qcif, QCIF_LUM_SIZE*9/8, MAX_USERS);
			confSendVideo(YUV9, yuv9qcif, QCIF_LUM_SIZE*9/8);
			size = QCIF_LUM_SIZE*9/8;
			GlobalFree(yuv9qcif);	
		}
	
		break;
	}

	intraFrame = (intraFrame + 1) % INTRA_GAP;

	GlobalFree(codedBuf);
	GlobalFree(yuv);	
	
	stop = GetTickCount();
	codeTime = stop - start;
	
	main_dlg->stat->frame(size);
}

BOOL confPlay(void)
{
	if(state == STOP)
	{
		if(!captureInit(main_dlg->m_hWnd, 500))
		{
			main_dlg->MessageBox("Your machine does not support video capturing", 
				NULL, MB_OK | MB_ICONEXCLAMATION);
			return FALSE;
		}
		else
		{
			frameNum = 0;
			confSendMessage(videoTagPlay, NULL, 0);
			captureGrab();
			test = TRUE;
			main_dlg->stat->start();
			state = PLAY;
		}
	}
	else
		return FALSE;

	return TRUE;
}

BOOL confStop(void)
{
	if(state == PLAY)
	{
		state = STOP;
		captureClose();
		confSendMessage(videoTagStop, NULL, 0);
		intraFrame = 0;
		main_dlg->stat->stop();
	}
	else
		return FALSE;

	return TRUE;
}

void confPanel(void)
{
	CPanelDlg *panel;
	BOOL restart=FALSE;
	int nResponse;

	if(state == PLAY)
	{
		confStop();
		restart = TRUE;
	}

	if((code == H261) && (H261IntraMode))
	{
		panel = new CPanelDlg(H261intra);
	}
	else
	{
		panel = new CPanelDlg(code);
	}

	nResponse = panel->DoModal();
	if (nResponse == IDOK)
	{
		if(panel->code == H261intra)
		{
			code = H261;
			H261IntraMode = TRUE;
			p64setIntraMode();
		}
		else
		{
			code = panel->code;
			p64resetIntraMode();
			H261IntraMode = FALSE;
		}

	}
	else if (nResponse == IDCANCEL)
	{
	}

	delete panel;

	if(restart)
		confPlay();
}

void connPanel(void)
{
	CConnDlg *conn;
	BOOL restart=FALSE;
	int nResponse;

	if(state == PLAY)
	{
		confStop();
		restart = TRUE;
	}

	conn = new CConnDlg(connection);

	nResponse = conn->DoModal();
	if (nResponse == IDOK)
	{
		conn->getConn(connection); 
		setConnection(connection);
	}
	else if (nResponse == IDCANCEL)
	{
	}

	delete conn;

	if(restart)
		confPlay();
}

void setConnection(char *conn)
{
	if(strcmp(conn, "Modem 28.8") == 0)
	{
		code = H263;
		limit = 4;
	}

	if(strcmp(conn, "ISDN-1 channel") == 0)
	{
		code = H261;
		H261IntraMode = FALSE;
		p64resetIntraMode();
		limit = 5;
	}

	if(strcmp(conn, "ISDN-2 channels") == 0)
	{
		code = H261;
		H261IntraMode = FALSE;
		p64resetIntraMode();
		limit = 10;
	}

	if(strcmp(conn, "LAN") == 0)
	{
		code = YUV9;
		limit = 10;
	}
}

void confClose(void)
{
	if(state == PLAY)
		captureClose();
	confSaveConfig();
	CloseHandle(hCodecEvent);
	displayQuit();
	TerminateThread(hThread, 0);
	CloseHandle(hThread);
	ncsAppDettach(ncsApp);
}

void confExit()
{
	DWORD wParam;
	
	wParam = BN_CLICKED * 65536 + IDM_EXIT;
	SendMessage(main_dlg->m_hWnd, WM_COMMAND, wParam, NULL); 
}

DWORD confTimeCtrlThread(LPDWORD lpdwParam)
{
	DWORD wParam;

	while(1)
	{
		Sleep(10000);
		if((state == PLAY) && (test))
		{
			wParam = UPDATE_RATE*65536;
			SendMessage(main_dlg->m_hWnd, WM_COMMAND, wParam, NULL); 
		}
	}
	return 0;
}

void confUpdateRate()
{
	if(code == H263)
		captureSetRate((double)codeTime * (double)2);
	else
		if(!H261IntraMode)
			captureSetRate((double)codeTime * (double)2);
		else
			captureSetRate((double)codeTime * (double)3);

	test = FALSE;
}

char *confGetUserString(int no)
{
	char user[20];
	char host[50];
	char *str;

	str = (char *)malloc(70);
	ncsAppGetPart(ncsApp, no, user, host);
	wsprintf(str, "%s@%s", user, host);

	return str;
}

void confSaveConfig()
{
	int len;
	char path[100];
	FILE *file;

	getStartPath(path);	
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, "video.cfg");

	file = fopen(path, "w");
	if(file != NULL)
	{
		fprintf(file, connection);
		fclose(file);
	}
}

int confRestoreConfig()
{
	int len;
	char path[100];
	FILE *file;

	getStartPath(path);	
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, "video.cfg");

	file = fopen(path, "r");
	if(file == NULL)
		return 0;
	else
	{
		fgets(connection, 20, file);
		fclose(file);
	}

	return 1;
}

void getStartPath(char *path)
{
	int i;
	char *tmp;
	int found = 0;
	char p[100];

	tmp = GetCommandLine();
	strcpy(p, tmp);
	i = strlen(p) - 1;
	while(!found && (i >= 0))
	{
		if(p[i] == '\\')
		{
			p[i] = 0;
			found = 1;
		}
		i--;
	}
	if(!found)
		strcpy(p, "");
	
	if(p[0] == '"')
		strcpy(path, &p[1]);
	else
		strcpy(path, p);

}

