//==========================================================
//
// Written by Lukasz Beca, modified by Grzegorz Lewandowski
//
//----------------------------------------------------------

#include "stdafx.h"
#include "tango.h"
//#include "serror.h"

// only for testing
#include <stdio.h>


//
// readAll (by Pawel Roman, ported to Windows by Grzegorz Lewandowski)
//
HRESULT readAll( SOCKET fd, char* pBuffer, size_t Length )
{
	int nResult;
	do
	{
		nResult = recv( fd, pBuffer, Length, 0 );
		if( nResult > 0 )
		{
			Length -= nResult;
			pBuffer = pBuffer + nResult;
		}
		else
		{
			if( nResult == SOCKET_ERROR )
			{
				return ERROR_READ_FAULT;
			}
			return ERROR_HANDLE_EOF;			
		}
	}
	while( Length > 0 );
	return S_OK;
}


//
// writeAll
//
HRESULT writeAll( SOCKET fd, char* pBuffer, size_t Length )
{
	int nResult;
	do
	{
		nResult = send( fd, pBuffer, Length, 0 );
		if( nResult > 0 )
		{
			Length -= nResult;
			pBuffer = pBuffer + nResult;
		}
		else
		{
			if( nResult == SOCKET_ERROR )
			{
				return ERROR_WRITE_FAULT;
			}
			return ERROR_HANDLE_EOF;
		}
	}
	while( Length > 0 );
	return S_OK;
}


//
// registerApp
//
SOCKET registerApp( int AID, int port )
{
	SOCKET sockfd;
	SOCKADDR_IN serv_addr;
	char *pBuffer;
	long lToSend;
	int nResult;
	char HostName[ 127 ];
	struct hostent *pHostInfo;

	if( gethostname( HostName, 127 ) == SOCKET_ERROR )
	{
		return INVALID_SOCKET;
	}
	
	if( ( pHostInfo = gethostbyname( HostName ) ) == NULL )
	{
		return INVALID_SOCKET;
	}

	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr = *(struct in_addr*)( pHostInfo->h_addr ); 
	serv_addr.sin_port = htons( port );

	if( ( sockfd = socket( AF_INET, SOCK_STREAM, 0 ) ) == INVALID_SOCKET )
	{
		return INVALID_SOCKET;
	}

	if( connect( sockfd, (PSOCKADDR)&serv_addr, sizeof( serv_addr ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	lToSend = AID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}
	return sockfd;
}


//
// sendEvent
//
HRESULT sendEvent( SOCKET sockfd, struct Event *pev )
{
	int buffer;
	buffer = htonl( pev->type );

	HRESULT hr;
	hr = writeAll( sockfd, (char*)&buffer, sizeof( int ) );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	buffer = htonl( pev->len );
	hr = writeAll( sockfd, (char*)&buffer, sizeof( int ) );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	hr = writeAll( sockfd, pev->data, pev->len );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	return S_OK;
}


//
// receiveEvent
//
struct Event* receiveEvent( SOCKET sockfd )
{
	HRESULT hr;
	char Buffer[ 4 ];
	int *a;
	int type;
	int len;
	char *data;
	struct Event *r; 
	
	hr = readAll( sockfd, Buffer, 4 );
	if( hr != S_OK )
	{
		return NULL;
	}
	a = (int*)Buffer;
	type = ntohl( *a );
	
	hr = readAll( sockfd, Buffer, 4 );
	if( hr != S_OK )
	{
		return NULL;
	}
	a = (int*)Buffer;
	len = ntohl( *a );
	
	data = new char[ len ];
	
	hr = readAll( sockfd, data, len );
	if( hr != S_OK )
	{
		delete[] data;
		return NULL;
	}
	r = createEvent( type, len, data );
	if( r == NULL )
	{
		return NULL;
	}
	
	delete[] data;
	return r;
}


//
// createEvent
//
struct Event* createEvent( int type, int len, char *data )
{
	int i;
	struct Event *Result;
	
	Result = NULL;
	Result = new struct Event;
	if( Result == NULL )
	{
		return NULL;
	}
	Result->type = type;
	Result->len = len;
	Result->data = NULL;
	Result->data = new char[ len ];
	if( Result->data == NULL )
	{
		return NULL;
	}

	for( i = 0; i < len; i ++ )
	{
		Result->data[ i ] = data[ i ];
	}
	
	return Result;
}


//
// destroyEvent
//
void destroyEvent( struct Event *pev )
{
	delete[] pev->data;
	delete pev;
	pev = NULL;
}


//
// getType
//
int getType( struct Event *pev )
{
	return pev->type;
}


//
// getLen
//
int getLen( struct Event *pev )
{
	return pev->len;
}


//
// getData
//
char *getData( struct Event *pev )
{
	int nLength;
	int i;
	char *pBuffer;

	nLength = getLen( pev );
	pBuffer = new char[ nLength ];

	for( i = 0; i < nLength; i ++ ) 
	{
		pBuffer[ i ] = pev->data[ i ];
	}
	
	return pBuffer;
}
