///////////////////////////////////////////////////////////////////////////
// 
// File						: display.cpp
//
// Description				: Video display functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-21-96
//
////////////////////////////////////////////////////////////////////////////
#include <vfw.h>
#include <wingdi.h>

#include "stdafx.h"
#include "h261.h"
#include "h263.h"
#include "convert.h"
#include "video.h"
#include "display.h"
#include "DisplayDlg.h"
#include "yuvrgb24.h"
#include "UserData.h"


DisplayData dpyData[MAX_USERS+1];
int check[MAX_USERS+1];
BITMAPINFO *bmpInfo;
DLGTEMPLATE dlgTemplate;
static Video *video;

#define COL256 1
#define TRUECOL 2
int mode=TRUECOL;

void displayInit(Video *v)
{
	int i;
	DEVMODE devm;
	
	video = v;

	dlgTemplate.style = WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE;
	dlgTemplate.cdit = 0;
	dlgTemplate.x = 100;
	dlgTemplate.y = 100;
	dlgTemplate.cx = 176;
	dlgTemplate.cy = 164;

	bmpInfo = (BITMAPINFO *)GlobalAlloc(GPTR, sizeof(BITMAPINFO));

	// BITMAPINFO structure initialization (video format)
	//bmpInfo->bmiColors = (RGBQUAD *)NULL;
	bmpInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmpInfo->bmiHeader.biWidth = 176; //176;
	bmpInfo->bmiHeader.biHeight = 144; //144;
	bmpInfo->bmiHeader.biPlanes = 1;
	bmpInfo->bmiHeader.biBitCount = 24;
	bmpInfo->bmiHeader.biCompression = BI_RGB;
	bmpInfo->bmiHeader.biSizeImage = 0;
	bmpInfo->bmiHeader.biXPelsPerMeter = 0;
	bmpInfo->bmiHeader.biYPelsPerMeter = 0;	
	bmpInfo->bmiHeader.biClrUsed = 0;
	bmpInfo->bmiHeader.biClrImportant = 0;	

	for(i=0; i<MAX_USERS+1; i++)
	{
		dpyData[i].first = 1;
		dpyData[i].wndActive = 0;
	}

	init_dither_tab();

	//i=0;
	//while(EnumDisplaySettings(NULL, i++, &devm));
	devm.dmBitsPerPel = GetDeviceCaps(CreateDC("DISPLAY", NULL, NULL, NULL), BITSPIXEL);
	if(devm.dmBitsPerPel <= 8)
		mode = COL256;
	else
		mode = TRUECOL;

}

void displayQuit(void)
{
	int i;

	for(i=0; i<MAX_USERS; i++)
	{
		if(dpyData[i].wndActive)
			displayClose(i);
	}
	GlobalFree(bmpInfo);
}

CDialog *displayOpen(DWORD user)
{
	char text[100];
	char *tmp;

	//wsprintf(text, "User %d", user);
	//MessageBox(NULL, text, "Info", MB_OK);

	// new display dialog
	dpyData[user].dpy = new CDisplayDlg();

	if(!dpyData[user].dpy->Create(IDD_DISPLAY))
		MessageBox(NULL, "Error creating window", "Error", MB_OK);

	if(user != MAX_USERS)
	{
		dpyData[user].dpy->SetWindowPos(&dpyData[user].dpy->wndTopMost, 200*user+100, 100, 176, 164, SWP_NOSIZE);
		
		tmp = video->GetUserString(user);
		dpyData[user].dpy->SetWindowText(tmp);
		free(tmp);
	}
	else
	{
		dpyData[user].dpy->ModifyStyle(WS_SIZEBOX, 0);
		dpyData[user].dpy->SetWindowText("Mirror");
		//dpyData[user].dpy->ShowWindow(SW_HIDE);
		((CDisplayDlg *)dpyData[user].dpy)->SetStatus("Audio Off");
	}

	if(mode == COL256)
		dpyData[user].dpy->ModifyStyle(WS_SIZEBOX, 0);

	dpyData[user].cDc = new CClientDC(dpyData[user].dpy);
	dpyData[user].cDc->SetMapMode(MM_TEXT);
	dpyData[user].hDc = dpyData[user].cDc->GetSafeHdc();

	SetStretchBltMode(dpyData[user].hDc, HALFTONE);	
	UnrealizeObject(GetCurrentObject(dpyData[user].hDc, OBJ_BRUSH));
	SetBrushOrgEx(dpyData[user].hDc, 0, 0, NULL);
	SelectObject(dpyData[user].hDc, GetCurrentObject(dpyData[user].hDc, OBJ_BRUSH));
		//GetStockObject(NULL_BRUSH));

	if(mode == COL256)
	{
		dpyData[user].hDraw = DrawDibOpen();

		if(!DrawDibBegin(dpyData[user].hDraw, dpyData[user].hDc, 
							176, 144, 
							&bmpInfo->bmiHeader, -1, -1, DDF_HALFTONE ))
			MessageBox(NULL, "Draw Dib begin error", "Error", MB_OK);
	}

	dpyData[user].wndActive = 1;

	return dpyData[user].dpy;
  }

void displayClose(DWORD user)
{
	if(dpyData[user].wndActive)
	{
		delete dpyData[user].cDc;
		dpyData[user].dpy->DestroyWindow();
		delete dpyData[user].dpy;
		if(mode == COL256)
			DrawDibClose(dpyData[user].hDraw);
		dpyData[user].wndActive = 0;
	}
}

void displayShow(DWORD user)
{
	if(dpyData[user].wndActive)
	{
		dpyData[user].dpy->ShowWindow(SW_SHOW);
	}
}

void displayHide(DWORD user)
{
	if(dpyData[user].wndActive)
	{
		dpyData[user].dpy->ShowWindow(SW_HIDE);
	}
}

void displaySetStatus(DWORD user, char *status)
{
	if(dpyData[user].wndActive)
	{
		((CDisplayDlg *)dpyData[user].dpy)->SetStatus(status);
	}
}

void displayYUV(unsigned char *data, int len, DWORD user)
{
	unsigned char *yuv;
	char *image;
	char text[100];
	int err;
	RECT rect;

	if(dpyData[user].wndActive)
	{
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);
	
		ConvertYUVtoRGB(data, &data[QCIF_LUM_SIZE], &data[QCIF_LUM_SIZE + QCIF_CR_SIZE],
						(unsigned char *)image,
						176, 144);

		
		dpyData[user].dpy->GetClientRect(&rect);

		if(mode == TRUECOL)
		{
			err = StretchDIBits(dpyData[user].hDc, 0, 0, rect.right, rect.bottom-40, 0, 0, 176, 144,
					image, bmpInfo, DIB_RGB_COLORS, SRCCOPY);

			if(err == GDI_ERROR)
				MessageBox(NULL, "GDI error", "Error", MB_OK);
		}

		if(mode == COL256)
		{
			DrawDibDraw(dpyData[user].hDraw, dpyData[user].hDc, 0, 0, 176, 144, 
				&bmpInfo->bmiHeader, image, 
				0, 0, 176, 144, DDF_HALFTONE);
		}
		
		GlobalFree(image);

		displayUpdateStats(user);
		
	}
}

void displayYUV9(unsigned char *data, int len, DWORD user)
{
	unsigned char *yuv;
	char *image;
	char text[100];
	RECT rect;
	int err;

	if(dpyData[user].wndActive)
	{
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);
		yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3/2);
	
		YVU9toYUV411(data, 
				yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
				176, 144);

		ConvertYUVtoRGB(yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
						(unsigned char *)image,
						176, 144);
	
		dpyData[user].dpy->GetClientRect(&rect);

		if(mode == TRUECOL)
		{
			err = StretchDIBits(dpyData[user].hDc, 0, 0, rect.right, rect.bottom-40, 0, 0, 176, 144,
					image, bmpInfo, DIB_RGB_COLORS, SRCCOPY);

			if(err == GDI_ERROR)
				MessageBox(NULL, "GDI error", "Error", MB_OK);
		}

		if(mode == COL256)
		{
			DrawDibDraw(dpyData[user].hDraw, dpyData[user].hDc, 0, 0, 176, 144, 
				&bmpInfo->bmiHeader, image, 
				0, 0, 176, 144, DDF_HALFTONE);
		}
	
		GlobalFree(image);
		GlobalFree(yuv);

		displayUpdateStats(user);
			
	}
}


void displayH261(void *data, int len, DWORD user)
{
	unsigned char *yuv;
	char *image;
	char text[100];
	RECT rect;
	int err;

	//wsprintf(text, "Data size %d, user %d", len, user);
	//MessageBox(NULL, text, "Info", MB_OK);

	if(dpyData[user].wndActive)
	{
		yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3/2);
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);

		WaitForSingleObject(video->hCodecEvent, INFINITE);
		ResetEvent(video->hCodecEvent);
		if(dpyData[user].first)
		{
			p64DecodeFrame((char *)data, len, (char *)yuv, 1, user);
			dpyData[user].first = 0;
		}
		else
			p64DecodeFrame((char *)data, len, (char *)yuv, 0, user);
		SetEvent(video->hCodecEvent);

	
		ConvertYUVtoRGB(yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
						(unsigned char *)image,
						176, 144);
	
		dpyData[user].dpy->GetClientRect(&rect);

		if(mode == TRUECOL)
		{
			err = StretchDIBits(dpyData[user].hDc, 0, 0, rect.right, rect.bottom-40, 0, 0, 176, 144,
					image, bmpInfo, DIB_RGB_COLORS, SRCCOPY);

			if(err == GDI_ERROR)
				MessageBox(NULL, "GDI error", "Error", MB_OK);
		}

		if(mode == COL256)
		{
			DrawDibDraw(dpyData[user].hDraw, dpyData[user].hDc, 0, 0, 176, 144, 
				&bmpInfo->bmiHeader, image, 
				0, 0, 176, 144, DDF_HALFTONE);
		}
	
		GlobalFree(yuv);
		GlobalFree(image);

		displayUpdateStats(user);

	}
}


void displayH263(void *data, int len, DWORD user)
{
	char *image;
	char text[100];
	unsigned char **pYUV;
	RECT rect;
	int err;

	//wsprintf(text, "Data size %d, user %d", len, user);
	//MessageBox(NULL, text, "Info", MB_OK);

	if(dpyData[user].wndActive)
	{
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);

		pYUV = H263decodeFrame((char *)data, len, user);
	
		ConvertYUVtoRGB(pYUV[0], pYUV[1], pYUV[2],
						(unsigned char *)image,
						176, 144);
	
		dpyData[user].dpy->GetClientRect(&rect);

		if(mode == TRUECOL)
		{
			err = StretchDIBits(dpyData[user].hDc, 0, 0, rect.right, rect.bottom-40, 0, 0, 176, 144,
					image, bmpInfo, DIB_RGB_COLORS, SRCCOPY);

			if(err == GDI_ERROR)
				MessageBox(NULL, "GDI error", "Error", MB_OK);
		}

		if(mode == COL256)
		{
			DrawDibDraw(dpyData[user].hDraw, dpyData[user].hDc, 0, 0, 176, 144, 
				&bmpInfo->bmiHeader, image, 
				0, 0, 176, 144, DDF_HALFTONE);
		}
	
		displayUpdateStats(user);
		
		GlobalFree(image);
	}
}

void displayUpdateStats(int user)
{
	check[user] = ++check[user] % 5;
	if(check[user] == 0)
		if(video->userdata[user] != 0)
		{
			((CDisplayDlg *)dpyData[user].dpy)->setKbps(video->userdata[user]->getKbps());
			((CDisplayDlg *)dpyData[user].dpy)->setFps(video->userdata[user]->getFps());
		}
	
}