#include "stdafx.h"

#include <mmsystem.h>

#include "BuenaVista.h"
#include "BuenaVistaDlg.h"
#include "Conference.h"
#include "AddUsers.h"
#include "ChatDlg.h"
#include "global.h"
#include "daemon.h"
#include "options.h"
#include "PersonalInfo.h"

extern Session session;

Conference::Conference(CBuenaVistaDlg *dlg)
{
	HANDLE hProcess;
	CPersonalInfo *info_dlg;
	
	hProcess = GetCurrentProcess();
	SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);

	main_dlg = dlg;
	restoreSettings();
	if((strcmp(options.user_name, "") == 0) || (strcmp(options.user_email, "") == 0))
	{
		info_dlg = new CPersonalInfo(&options);
		info_dlg->DoModal();
		delete info_dlg;
	}

	iaminvited = 0;
	conferenceExists = 0;
	activePartsNum = 0;

	initDaemon(this);

	//////////////////////////////////
	// Daemon thread creation
	hDaemonThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) daemonThread,	// thread function 
        this,									// argument to thread function 
        0,										// use default creation flags 
        &dwDaemonThreadId);						// returns the thread identifier 
	if (hDaemonThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	
	SetThreadPriority(hDaemonThread, THREAD_PRIORITY_LOWEST);


	ncsSessionInit();

	server = 0;
	//server = ncsServerInit(options.server_name);
	if(server)
		ncsServerAdd();

	main_dlg->SetStatus("Conferencing panel ready to use");
}

Conference::~Conference()
{
	if(server && !conferenceExists)
		ncsServerDelete();
	ncsSessionQuit();
	TerminateThread(hDaemonThread, 0);
	CloseHandle(hDaemonThread);
	//Sleep(1000);
	quitDaemon();
	saveSettings();
}

DWORD Conference::daemonThread(LPDWORD data)
{
	runDaemon();

	return 1;
}

int Conference::saveSettings()
{
	char path[100];
	FILE *file;
	char *tmp;
	
	this->getStartPath(path);
	if(strcmp(path, "") == 0)
		strcat(path, "BuenaVista.cfg");
	else
		strcat(path, "\\BuenaVista.cfg");

	if(path[0] == '"')
		tmp = &path[1];
	else
		tmp = path;

	//MessageBox(NULL, tmp, "Info", MB_OK);
	file = fopen(tmp, "wb");
	if(file == NULL)
		return 0;	
	else
		if(fwrite(&options, sizeof(Options), 1, file) < 1)
			return 0;

	return 1;
}

int Conference::restoreSettings()
{
	char path[100];
	FILE *file;
	char *tmp;
	
	this->getStartPath(path);
	if(strcmp(path, "") == 0)
		strcat(path, "BuenaVista.cfg");
	else
		strcat(path, "\\BuenaVista.cfg");

	if(path[0] == '"')
		tmp = &path[1];
	else
		tmp = path;

	//MessageBox(NULL, tmp, "Info", MB_OK);
	file = fopen(tmp, "rb");
	if(file == NULL)
		standardSettings();
	else
		if(fread(&options, sizeof(Options), 1, file) < 1)
			standardSettings();

	return 1;
}

void Conference::standardSettings()
{
	strcpy(options.connection, "Modem 28.8");
	strcpy(options.server_name, "kopernik.npac.syr.edu");
	strcpy(options.user_name, "");
	strcpy(options.user_email, "");
	options.autoAccept = TRUE;

	options.audio.compression = GSM;
	options.audio.autoSwitch = FALSE;
	options.audio.manualSwitch = FALSE;
	options.audio.speakerTreshold = 20;
	options.audio.micTreshold = 20;
	options.audio.mode = HALF_DUPLEX;
	options.audio.volume = 50;
	options.audio.micGain = 50;
	options.audio.mute = FALSE;
	options.audio.micOff = FALSE;
	
	options.video.device = 0;
	options.video.compression = H263;
	options.video.intraMode = FALSE;
	options.video.limit = 5;
	options.video.maxlimit = 5;
	options.video.recOff = FALSE;
	
	options.audio.audioOut = NULL;

}

///////////////////////////////////////////////////////////////////////////////
//
// BuenaVista callbacks
//
///////////////////////////////////////////////////////////////////////////////
void Conference::invited(char *user, char *host)
{
	char text[200];

	strcpy(inviting_user, user);
	strcpy(inviting_host, host);
	iaminvited = 1;
	wsprintf(text, "INVITATION from %s@%s", user, host);
	main_dlg->SetStatus(text);
	if(options.autoAccept)
	{
		accept();
	}
	else
	{
		main_dlg->EnableInvitationButtons();
	
		this->startRing();
	}
}

void Conference::accepted(char *user, char *host)
{
	char text[100];

	wsprintf(text, "%s@%s accepted", user, host);
	main_dlg->SetStatus(text);
}

void Conference::rejected(char *user, char *host)
{
	char text[100];

	wsprintf(text, "%s@%s rejected", user, host);
	main_dlg->SetStatus(text);
}

void Conference::connected(char *user, char *host)
{
	char text[100];

	wsprintf(text, "%s@%s received invitation", user, host);
	main_dlg->SetStatus(text);
}

void Conference::timeout(char *user, char *host)
{
	char text[100];

	wsprintf(text, "%s@%s not present", user, host);
	main_dlg->SetStatus(text);
}

void Conference::newUser(char *user, char *host)
{
	main_dlg->SetStatus("New participant joins");
	main_dlg->Add2UserList(user, host);
	if(conferenceExists == 1)
	{
		main_dlg->startAudio();
	}
	conferenceExists++;
}

void Conference::leftUser(char *user, char *host)
{
	char text[200];

	main_dlg->DeleteFromUserList(user, host);
	wsprintf(text, "%s@%s left", user, host);
	main_dlg->SetStatus(text);
}

////////////////////////////////////////////////////////////////////////
//
// Members management
//
////////////////////////////////////////////////////////////////////////
void Conference::parseIPaddress(char *IP, char *name, char *host)
{
  char *ptr, *pos;
  int len;

  ptr = IP;
  pos = strchr(ptr, 64); /* code of @ */
  if(pos != NULL)
  {
	len = (int)pos - (int)ptr;
	memcpy(name, ptr, len);
	name[len]='\0';
	ptr += len+1;
	memcpy(host, ptr, strlen(IP)-len-1);
	host[strlen(IP)-len-1]='\0';
  }
  else
  {
	  strcpy(name, ptr);
	  host = NULL;
  }
} /* parseIPaddress() */

void Conference::addUser(char *member)
{
	NCSparticipant part;
	Participant mbr;

	if(!conferenceExists)
	{
		ncsSessionCreate();
	}
	parseIPaddress(member, mbr.user, part.host);		
	
	if(part.host != NULL)
		if((mbr.host = getHostID(part.host)) != 0)
			ncsSessionAddMember(&mbr);
}

void Conference::addIPuser(Participant *part)
{
	if(!conferenceExists)
	{
		ncsSessionCreate();
	}
	
	ncsSessionAddMember(part);
}
/////////////////////////////////////////////////////////////////////////
//
// Ringing
//
/////////////////////////////////////////////////////////////////////////
void Conference::startRing()
{
	//////////////////////////////////
	// Ringing thread creation
	hRingThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) ringingThread, // thread function 
        this,									// argument to thread function 
        0,										// use default creation flags 
        &dwRingThreadId);						// returns the thread identifier 
	if (hRingThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
}

void Conference::stopRing()
{
	TerminateThread(hRingThread, 0);
	//main_dlg->resetRing();
}

DWORD Conference::ringingThread(LPDWORD data)
{
	char path[100];
	int i;
	
	((Conference *)data)->getStartPath(path);
	if(strcmp(path, "") == 0)
		strcat(path, "RINGIN.WAV");
	else
		strcat(path, "\\RINGIN.WAV");
	
	for(i=0; i<10; i++)
	{
		//main_dlg->setRing();
		PlaySound(path, NULL, SND_SYNC | SND_NODEFAULT);
		//main_dlg->resetRing();
		Sleep(2000);
	}

	return 1;
}


void Conference::accept()
{
	this->stopRing();
	main_dlg->EnableInvitationButtons(FALSE);
	//Sleep(500);
	ncsSessionJoin();
}

void Conference::reject()
{
	ncsSessionReject();
	main_dlg->EnableInvitationButtons(FALSE);
	this->stopRing();
}

void Conference::addUserCallback()
{
	CAddUsers *dlg;
	char *users;

	if(server)
	{
		users = ncsServerUsers();
		if(users != NULL)
		{
			this->serverUsers(users);
			free(users);
		}
	}

	dlg = new CAddUsers(this);

	int nResponse = dlg->DoModal();

	if (nResponse == IDOK)
	{
	}	
	else if (nResponse == IDCANCEL)
	{
	}

	delete dlg;
}

void Conference::serverUsers(void *data)
{
	ServerData *users;
	int i;

	users = (ServerData *)data;
	activePartsNum = ntohl(users->numMembers);
	for(i=0; i<activePartsNum; i++)
	{
		strcpy(activeParts[i].user, users->parts[i].user);
		getHostName(users->parts[i].host, activeParts[i].host);
	}
}

Session * Conference::getSessionInfo()
{
	return &session;
}


void Conference::getStartPath(char *path)
{
	int i;
	char *tmp;
	int found = 0;

	tmp = GetCommandLine();
	strcpy(path, tmp);
	i = strlen(path) - 1;
	while(!found && (i >= 0))
	{
		if(path[i] == '\\')
		{
			path[i] = 0;
			found = 1;
		}
		i--;
	}
	if(!found)
		strcpy(path, "");
}

void Conference::resetSession()
{
	ncsSessionReset();
	if(server)
		ncsServerAdd();
	conferenceExists = 0;
}

void Conference::appMessage(int usr, int no, int tag, char *msg, int len)
{
	if(no == 6666)
		((CChatDlg *)main_dlg->chat)->message(usr, tag, msg, len);
}
