///////////////////////////////////////////////////////////////////////////
// 
// File						: video.cpp
//
// Description				: video control functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 10-02-97
//
////////////////////////////////////////////////////////////////////////////
#include <vfw.h>

#include "stdafx.h"
#include "BuenaVista.h"
#include "BuenaVistaDlg.h"
#include "display.h"
#include "video.h"
#include "global.h"
#include "VideoProtocol.h"
#include "h261.h"
#include "h263.h"
#include "capture.h"
#include "convert.h"
#include "UserData.h"
#include "AudioStat.h"
#include "options.h"
#include "conference.h"

extern FILE *clog;

void Video::Exit(Video *video)
{
	DWORD wParam;
	
	//wParam = BN_CLICKED * 65536 + IDM_EXIT;
	//video->main_dlg->SendMessage(WM_COMMAND, wParam, NULL); 
	video->main_dlg->SendMessage(WM_CLOSE, 0, NULL); 
}

Video::Video(CBuenaVistaDlg *dlg, Session *session)
{
	DWORD wParam;
	int i;
	
	main_dlg = dlg;
	options = &(((Conference *)main_dlg->conference)->options.video);
	frameNum = 0;
	limit = 4;
	intraFrame = 0;
	codeTime=500;
	seq = 0;
	for(i=0; i<MAX_USERS+1; i++)
	{
		userdata[i] = NULL;
		decodeTime[i] = 0;
    }


	// Start up NCS
	ncsApp = (ncsApplication *)ncsAppAttach(VIDEO_PLUGIN_PORT, session);
	if(ncsApp == NULL)
	{
		this->Exit(this);
		return;
	}

	// Create reading thread
	hThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) this->ReadThread,	// thread function 
			this,									// argument to thread function 
			0,										// use default creation flags 
			&dwThreadId);							// returns the thread identifier 
	
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

	// Create reading thread
	hCapThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) this->CaptureThread,	// thread function 
			this,									// argument to thread function 
			0,										// use default creation flags 
			&dwThreadId);							// returns the thread identifier 
	
	if (hCapThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

	p64InitCodec();
	p64InitEncoder();
	p64InitDecoder();

	H263initEncoder(176, 144);
	H263initDecoder();

	displayInit(this);

	hCodecEvent = CreateEvent(NULL, TRUE, TRUE, "codec_event");

	state = STOP;
	code = H263;
	intra = 1;
	H261IntraMode = FALSE;
	captureOn = FALSE;

	for(i=0; i<MAX_MEMBERS; i++)
		first[i] = TRUE;

	if(!capture.init(main_dlg->m_hWnd, options->device))
	{
		main_dlg->MessageBox("Your machine does not support video capturing", 
			NULL, MB_OK | MB_ICONEXCLAMATION);
	}

	this->SendMessage(videoTagNewMember, NULL, 0);
}

Video::~Video()
{
	if(state == PLAY)
	{
		capture.stop();	
	}

	capture.close();
	TerminateThread(hThread, 0);
	CloseHandle(hThread);
	TerminateThread(hCapThread, 0);
	CloseHandle(hCapThread);
	CloseHandle(hCodecEvent);
	displayQuit();
	ncsAppDettach(ncsApp);
}

DWORD Video::ReadThread(LPDWORD lpdwParam)
{
	struct timeval timeout;
	fd_set set;
	RTPheader header;
	int size;
	int sockfd;
	char *data;
	char *msg;
	Video *tmp = (Video *)lpdwParam;

	timeout.tv_sec = 0;
	timeout.tv_usec = 100000;

	while(1)
	{
		sockfd = ncsAppGetFD(tmp->ncsApp, &set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsAppProcessIncoming(tmp->ncsApp, &set);
			size = ncsAppGetData(tmp->ncsApp, &set, &data);
			if(size > 0)
			{
				size = size - RTP_HEADER_SIZE;
				readRTPheader(&header, data);
				msg = data + RTP_HEADER_SIZE;
				tmp->HandleData(&header, msg, size);
				free(data);
				SetFocus(tmp->main_dlg->m_hWnd);
			}
		}
	}
	return 1;
}

void Video::HandleData(RTPheader *header, void *data, int len)
{
	char buffer[1600];
	VideoPluginFrame *frame;
	DWORD wParam;
	int size;
	DWORD handle, tag;
	DWORD start, stop;
	
	start = GetTickCount();

	handle = header->ssrc;
	tag = header->pt;

	if(userdata[handle] == NULL)
		userdata[handle] = new UserData(header, len);
	else
		if(!userdata[handle]->frame(header, len))
			return;

	((CAudioStat *)main_dlg->audioStat)->Vupdate(handle, userdata[handle]);

	// MessageBox(NULL, "Get audio data", "Info", MB_OK);
	switch (tag)
	{
	case H261:
		frame = (VideoPluginFrame *)data;		
		if((frame->num < lastMsgNum[handle]) && ((lastMsgNum[handle] - frame->num) < 100))
		{
			// misplaced packet
		}
		else
		{
			lastMsgNum[handle] = frame->num;
			size = len - sizeof(VideoPluginFrame) + 1;
			displayH261(frame->data, size, handle);
		}
		break;
    
	case H263:
		frame = (VideoPluginFrame *)data;
		if((frame->num < lastMsgNum[handle]) && ((lastMsgNum[handle] - frame->num) < 100))
		{
			// misplaced packet
		}
		else
		{
			lastMsgNum[handle] = frame->num;
			size = len - sizeof(VideoPluginFrame) + 1;
			displayH263(frame->data, size, handle);
		}
		break;
	
	case YUV9:
		frame = (VideoPluginFrame *)data;
		if((frame->num < lastMsgNum[handle]) && ((lastMsgNum[handle] - frame->num) < 100))
		{
			// misplaced packet
		}
		else
		{
			lastMsgNum[handle] = frame->num;
			size = len - sizeof(VideoPluginFrame) + 1;
			displayYUV9((unsigned char *)frame->data, size, handle);
		}
		break;
	
	case videoTagExit:
        // exit plugin 
		break;

    case videoTagConfirm:
		break;

    case videoTagPlay:
		lastMsgNum[handle] = 0;
		this->SendMessage(videoTagConfirm, NULL, 0);
		if(first[handle])
		{
			//displayOpen(handle);
			wParam = CREATE_DISPLAY*65536 + handle;
			main_dlg->SendMessage(WM_COMMAND, wParam, NULL); 
			first[handle] = FALSE;
		}
		break;

    case videoTagNewMember:
        if(state == PLAY)
		{
			intraFrame = 0;
	        ncsAppSendData(ncsApp, handle, videoTagPlay, NULL, 0);
		}
		break;

    case videoTagStop:
		wParam = DESTROY_DISPLAY*65536 + handle;
		main_dlg->SendMessage(WM_COMMAND, wParam, NULL); 
		first[handle] = TRUE;
		decodeTime[handle] = 0;
		break;

    case videoTagFrameDecoded:
		break;

    default:
		break;
	}

	stop = GetTickCount();

	if((tag == H261) && (tag == H263) && (tag == YUV9))
		decodeTime[handle] = start - stop;
}

void Video::SendMessage(DWORD tag, void *data, int len)
{
	ncsAppSendData(ncsApp, -1, tag, data, len);
}


void Video::SendVideo(int tag, char *buf, int len)
{
	VideoPluginFrame *frame;
	int size;

	size = sizeof(VideoPluginFrame) + len - 1;
	frame = (VideoPluginFrame *)GlobalAlloc(GPTR, size);
	//gettimeofday(&frame->time);
	frameNum = (frameNum + 1) % 256;
	frame->num = (unsigned char)frameNum;
	memcpy(frame->data, buf, len);
	this->SendMessage(tag, frame, size);
	GlobalFree(frame);
}

void Video::CodeImage(char *yuv9, int dx, int dy, int conv)
{
	char *codedBuf;
	int size;
	DWORD start, stop;
	unsigned char *yuv;
	char *yuv9qcif;
	char *buf;


	start = GetTickCount();
	
	yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_YUV_SIZE);
	codedBuf = (char *)GlobalAlloc(GPTR, 20000);

	switch(options->compression)
	{
	case H261:
		if(conv)
		{
			YVU9toYUV411((unsigned char *)yuv9, 
					yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
					dx, dy);
			buf = (char *)yuv;
		}
		else
			buf = yuv9;

		displayYUV((unsigned char *)buf, QCIF_YUV_SIZE, MAX_USERS);

		WaitForSingleObject(hCodecEvent, INFINITE);
		ResetEvent(hCodecEvent);
	
		if(options->intraMode)
			p64setIntraMode();

		if(!intraFrame)
		{
			size = p64EncodeIntraFrame((char *)buf, codedBuf);
		}
		else
			size = p64EncodeFrame((char *)buf, codedBuf);
	
		SetEvent(hCodecEvent);
	
		this->SendVideo(H261, codedBuf, size);

		break;
	
	case H263:
		if(conv)
		{
			YVU9toYUV411((unsigned char *)yuv9, 
					yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
					dx, dy);
			buf = (char *)yuv;
		}
		else
			buf = yuv9;

		displayYUV((unsigned char *)buf, QCIF_YUV_SIZE, MAX_USERS);

		if(!intraFrame)
		{
			size = H263codeIntraFrame((unsigned char *)buf, codedBuf);
			intra = 0;
		}
		else
			size = H263codeFrame((unsigned char *)buf, codedBuf);

		this->SendVideo(H263, codedBuf, size);
	
		break;

	case YUV9:
		if(!conv)
		{
			options->compression = H261;
			options->intraMode = TRUE;
			p64setIntraMode();
		}
		else
		{
			yuv9qcif = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*9/8);
			resizeYUV9((char *)yuv9, dx, dy, (char *)yuv9qcif, 176, 144);
			displayYUV9((unsigned char *)yuv9qcif, QCIF_LUM_SIZE*9/8, MAX_USERS);
			this->SendVideo(YUV9, yuv9qcif, QCIF_LUM_SIZE*9/8);
			size = QCIF_LUM_SIZE*9/8;
			GlobalFree(yuv9qcif);	
		}
	
		break;
	}

	intraFrame = (intraFrame + 1) % INTRA_GAP;

	GlobalFree(codedBuf);
	GlobalFree(yuv);	
	
	StatUpdate(size);

	stop = GetTickCount();
	codeTime = stop - start;
}

BOOL Video::Play(void)
{
	if(state == STOP)
	{
			frameNum = 0;
			this->SendMessage(videoTagPlay, NULL, 0);
			capture.start();
			state = PLAY;
	}
	else
		return FALSE;

	return TRUE;
}

BOOL Video::Stop(void)
{
	if(state == PLAY)
	{
		state = STOP;
		capture.stop();
		this->SendMessage(videoTagStop, NULL, 0);
		intraFrame = 0;
	}
	else
		return FALSE;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
//
// Main video capturing thread
//
DWORD Video::CaptureThread(LPDWORD lpdwParam)
{
	Video *vd = (Video *)lpdwParam;
	unsigned char *frame;
	int type, dx, dy, time, i;
	unsigned char *tmp;
	int sleepTime;

	while(1)
	{
		if(vd->state == PLAY)
		{
			if(vd->capture.getFrame(&frame, &type, &dx, &dy))
			{
				switch(type)
				{
				case BI_RGB:
					tmp = new unsigned char[3*176*144/2];
					RGB24toYUV(frame, tmp, &tmp[25344], &tmp[25344+6336], dx, dy);
					vd->CodeImage((char *)tmp, 176, 144, 0);	
					delete tmp;
					break;

				case BI_YUV9:
					//tmp = new unsigned char[5*176*144/4];
					//resizeYUV9((char *)frame, dx, dy, (char *)tmp, 176, 144);
					vd->CodeImage((char *)frame, dx, dy, 1);
					//delete tmp;
					break;
				}
			}

			delete frame;
		}
		
		sleepTime = 0;
		for(i=0; i<MAX_USERS; i++)
			sleepTime += vd->decodeTime[i];
		sleepTime *= 1.5;
		sleepTime += 0.2 * vd->codeTime;
		sleepTime += 20;
		if(2*vd->codeTime + sleepTime < 1000/vd->options->limit)
			sleepTime = (1000/vd->options->limit) - 2*vd->codeTime;
/*
#ifdef _DEBUG
		if(clog != NULL)
			fprintf(clog, "Capture thread sleep %d\n", sleepTime);
#endif
*/
		if((sleepTime <= 0) || (sleepTime > 1000))
			sleepTime = 1000;
		Sleep(sleepTime);

	}

	return 0;
}

char *Video::GetUserString(int no)
{
	char user[20];
	char host[50];
	char *str;

	str = (char *)malloc(70);
	ncsAppGetPart(ncsApp, no, user, host);
	wsprintf(str, "%s@%s", user, host);

	return str;
}

void Video::SaveConfig()
{
	int len;
	char path[100];
	char name[20] = "NCS_PATH";
	FILE *file;

	len = GetEnvironmentVariable(name, path, 80);
	strcat(path, "\\NCSvideo\\video.cfg");

	file = fopen(path, "w");
	if(file != NULL)
	{
		fprintf(file, connection);
		fclose(file);
	}
}

int Video::RestoreConfig()
{
	int len;
	char path[100];
	char name[20] = "NCS_PATH";
	FILE *file;

	len = GetEnvironmentVariable(name, path, 80);
	strcat(path, "\\NCSvideo\\video.cfg");

	file = fopen(path, "r");
	if(file == NULL)
		return 0;
	else
	{
		fgets(connection, 20, file);
		fclose(file);
	}

	return 1;
}


void Video::StatUpdate(int size)
{
	RTPheader header;

	header.seq = seq;
	seq = (seq + 1) % 10000;

	if(userdata[MAX_USERS] == NULL)
		userdata[MAX_USERS] = new UserData(&header, size);
	else
		if(!userdata[MAX_USERS]->frame(&header, size))
			return;
}

void Video::Init()
{
	this->SendMessage(videoTagNewMember, NULL, 0);
}
