///////////////////////////////////////////////////////////////////////////
// 
// File						: video.h
//
// Description				: video functions header
//
// Author					: Tomasz Stachowiak
//
// Date						: 10-10-97
//
////////////////////////////////////////////////////////////////////////////

#ifndef _VIDEO_H
#define _VIDEO_H

#define MAX_USERS 20
#define INTRA_GAP 10
#define UPDATE_RATE 15865

#include "BuenaVista.h"
#include "BuenaVistaDlg.h"
#include "capture.h"
#include "global.h"
#include "UserData.h"
#include "options.h"

class Video
{
public:
	void Init();
	void StatUpdate(int size);
	CBuenaVistaDlg *main_dlg;		// main dlg
	ncsApplication *ncsApp;		// NCS application structure
	
	HANDLE hThread;
	DWORD dwThreadId;
	
	int state;					// encoding information
	int code;
	int intra;
	BOOL H261IntraMode;
	BOOL first[MAX_MEMBERS];	
	int frameNum;
	int lastMsgNum[MAX_MEMBERS];
	int intraFrame;
	char connection[20];
	int limit;
	VideoOptions *options;
	UserData *userdata[MAX_USERS+1];
	HANDLE hCodecEvent;
	Capture capture;
	
	Video(CBuenaVistaDlg *dlg, Session *session);
	~Video(void);
	BOOL Play(void);
	BOOL Stop(void);
	char *GetUserString(int no);

private:
	int seq;
	BOOL captureOn;
	DWORD codeTime;			// rate control informations
	int decodeTime[MAX_MEMBERS];
	HANDLE hCapThread;
	DWORD dwCapThreadId;
	BOOL test;


	void HandleData(RTPheader *header, void *data, int len);
	void SaveConfig();
	int RestoreConfig();
	static DWORD ReadThread(LPDWORD lpdwParam);
	static DWORD CaptureThread(LPDWORD lpdwParam);
	static void Exit(Video *video);
	void SendMessage(DWORD tag, void *frame, int len);
	void SendVideo(int tag, char *buf, int len);
	void CodeImage(char *yuv, int dx, int dy, int conv);

};

#endif
