///////////////////////////////////////////////////////////////////////////
// 
// File						: Conference.h
//
// Description				: Conference functions declarations
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-15-96
//
////////////////////////////////////////////////////////////////////////////
#ifndef _AUDIO_H
#define _AUDIO_H

#include "global.h"
#include "adpcm.h"
#include "gsm.h"
#include "DSaudio.h"
#include "DScapture.h"
#include "UserData.h"
#include "options.h"

#define		AUDIO_PLUGIN_PORT      12567
#define         VERSION      		1

#define MAX_USERS 100

#define ADPCM 13
#define GSM 14
#define PCM 15
#define SPECIAL 16
#define STOP 19
#define INIT 18
#define NEW 17

#define DUPLEX 20
#define HALF_DUPLEX 21

#define TRESHOLD 2000

#define BASIC_AUDIO_SIZE 160	/* samples */
#define AUDIO_CHUNKS_NO 5

typedef struct _audioMessage
{
  char buf[400];
}  audioPluginMessageStructure;



typedef enum _states
{
	stateMic,
	stateSpeaker,
	stateBoth
} States;

typedef enum _state_input
{
	Abut,
	Sbut,
	sig,
	Nsig,
	Mbut
} StateInput;

class Audio
{
private:
	void stateControl(StateInput in);
	void HDstateControl(StateInput in);
	void FDstateControl(StateInput in);
	void setBoth();
	void setSpeaker();
	void setMic();
	void checkTimer();
	short audioGetLevel(char *buffer);
	States state;
	int code_type;
	int tmp_code_type;
	HANDLE hThread;
	DWORD dwThreadId;
	struct adpcm_state adpcmstates[MAX_USERS];
	struct adpcm_state adpcmsendstate;
	int mode;
	int first[MAX_USERS];
	CBuenaVistaDlg *main_dlg;
	int silence_no;
	int silence_mic;
	int signal;
	gsm gsmHandle;
	int current_sender;
	int treshold;
	
	DSmultiAudio dsaudio;
	DScapture dscap;

	void HandleData(RTPheader *header, void *data, int len);

public:
	void setManual(int check);
	int mic_treshold;
	void setMicTreshold(int level);
	AudioOptions * options;
	UserData *userdata[MAX_USERS];
	ncsApplication *ncsApp;

	void codePCM();
	UINT timer;
	void setTalk(int check);
	void setAuto(int check);
	Audio(CBuenaVistaDlg *dlg, Session *session);
	int hardwareInit();
	static DWORD ReadThread(LPDWORD lpdwParam);
	void SendData(char *data);
	void StartSending(void);
	void StopSending(void);
	void codeADPCM(void);
	void codeGSM(void);
	void setTreshold(UINT level);
	~Audio(void);
	VOID static CALLBACK TimerCallback(
		HWND hwnd,	// handle of window for timer messages 
		UINT uMsg,	// WM_TIMER message
		UINT idEvent,	// timer identifier
		DWORD dwTime 	// current system time
	);
	
protected:
	int automatic;
	int manual;
	int talk;
};

#endif
