// DScapture.cpp: implementation of the DScapture class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <dsound.h>

#include "stdafx.h"
#include "BuenaVista.h"
#include "BuenaVistaDlg.h"
#include "Audio.h"
#include "DScapture.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef _DEBUG
static FILE *log;
#endif

int DScapture::init(void *aud)
{
	HRESULT hr;
	DSCBUFFERDESC              dscbd;
	WAVEFORMATEX               wfx =
		{WAVE_FORMAT_PCM, 1, 8000, 16000, 2, 16, 0};
		// wFormatTag, nChannels, nSamplesPerSec, mAvgBytesPerSec,
		// nBlockAlign, wBitsPerSample, cbSize
	DSBPOSITIONNOTIFY  rgdsbpn[PORTIONS_NO];
	WAVEFORMATEX rft;
	int i;
	DWORD tmp;
	char text[100];

#ifdef _DEBUG
	log = fopen("DScapture.log", "w");
#endif

	audio = aud;
	
	hr = DirectSoundCaptureCreate(NULL, &pDSC, NULL);
	if (hr != DS_OK)
		return 0;

	dscbd.dwSize = sizeof(DSCBUFFERDESC);
	dscbd.dwFlags = 0;
	dscbd.dwBufferBytes = 1600 * PORTIONS_NO;
	dscbd.dwReserved = 0;

	dscbd.lpwfxFormat = &wfx;
 
	pDSCB = NULL;
 
	hr = pDSC->CreateCaptureBuffer(&dscbd, &pDSCB, NULL);
	if (hr != DS_OK)
		return 0;

/*
	pDSCB->GetFormat(&rft, sizeof(WAVEFORMATEX), &tmp);
	wsprintf(text, "Format: %d samples/sec\n", rft.nSamplesPerSec);
	MessageBox(NULL, text, "Info", MB_OK);
*/
	for (i = 0; i < PORTIONS_NO; i++)
    {
        rghEvent[i] = CreateEvent(NULL, FALSE, FALSE, NULL);
        if (NULL == rghEvent[i]) 
			return 0;

		rgdsbpn[i].dwOffset = i*1600;
		rgdsbpn[i].hEventNotify = rghEvent[i];
    }
	
	hr = pDSCB->QueryInterface(IID_IDirectSoundNotify, (void **)&lpDsNotify);
	if (hr != DS_OK)
		return 0;

	// Create notifications.
	hr = lpDsNotify->SetNotificationPositions(PORTIONS_NO, rgdsbpn);
	if (hr != DS_OK)
		return 0;

    capture = TRUE;
	// create capturing thread
    hThread = CreateThread( 
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) captureThread, // thread function 
        this,									// argument to thread function 
        0,										// use default creation flags 
        &dwThreadId);							// returns the thread identifier 
	if (hThread == NULL) 
	{
		return 0;
	}

	return 1;
}

void DScapture::close()
{
	capture = FALSE;
	if(lpDsNotify != NULL)
		lpDsNotify->Release();
	if(pDSCB != NULL)
		pDSCB->Release();
	if(pDSC != NULL)
		pDSC->Release();

#ifdef _DEBUG
	fclose(log);
#endif
}

int DScapture::start()
{
	HRESULT hr;

	if(pDSCB != NULL)
		hr = pDSCB->Start(DSCBSTART_LOOPING);
	if(hr != DS_OK)
		return 0;

	return 1;
}

void DScapture::stop()
{
	HRESULT hr;
	
	if(pDSCB != NULL)
		pDSCB->Stop();
}

DWORD DScapture::captureThread(LPDWORD lpdwParam)
{
	int no;
	LPVOID	lpvPtr1;  
	DWORD	dwBytes1;  
	LPVOID	lpvPtr2;  
	DWORD	dwBytes2;  
	char buffer[1600];
	DScapture *cap = (DScapture *)lpdwParam;
	int pos;
	DWORD curpos, readpos;
	int hr;

	while(cap->capture)
	{
		no = WaitForMultipleObjects(PORTIONS_NO, cap->rghEvent, FALSE, INFINITE);
		pos = ((PORTIONS_NO + no - 1) % PORTIONS_NO) * 1600;

		cap->pDSCB->GetCurrentPosition(&curpos, &readpos);

#ifdef _DEBUG
		fprintf(log, "Capture: get pos = %d, current pos = %d, read pos = %d\n", pos, curpos, readpos);
#endif
	
		hr = cap->pDSCB->Lock(pos, 1600, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, 0);
		
		CopyMemory(buffer, lpvPtr1, dwBytes1);
		if(dwBytes2 > 0)
			CopyMemory(buffer + dwBytes1, lpvPtr2, dwBytes2);
		
		hr = cap->pDSCB->Unlock(lpvPtr1, dwBytes1, lpvPtr2, dwBytes2);

		((Audio *)cap->audio)->SendData(buffer);
	}

	return 1;
}