// AudioStat.cpp : implementation file
//
#include "stdafx.h"
#include "BVtango.h"
#include "AudioStat.h"
#include "UserData.h"
#include "ncs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAudioStat dialog


CAudioStat::CAudioStat(Session *ses, CWnd* pParent /*=NULL*/)
	: CDialog(CAudioStat::IDD, pParent)
{
	int i;
	
	//{{AFX_DATA_INIT(CAudioStat)
	//}}AFX_DATA_INIT

	session = ses;
	current = -1;
	check = 0;
	for(i=0; i<MAX_MEMBERS; i++)
		memberPos[i] = -1;
}


void CAudioStat::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAudioStat)
	DDX_Control(pDX, IDC_VREC, m_Vrec);
	DDX_Control(pDX, IDC_VLOST, m_Vlost);
	DDX_Control(pDX, IDC_VBAND, m_Vbandwidth);
	DDX_Control(pDX, IDC_FPS, m_Fps);
	DDX_Control(pDX, IDC_TOTAL, m_Total);
	DDX_Control(pDX, IDC_REC, m_Received);
	DDX_Control(pDX, IDC_PART_LIST, m_PartsList);
	DDX_Control(pDX, IDC_LOST, m_Lost);
	DDX_Control(pDX, IDC_LATE, m_Late);
	DDX_Control(pDX, IDC_LAT, m_Latency);
	DDX_Control(pDX, IDC_BAND, m_Bandwidth);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAudioStat, CDialog)
	//{{AFX_MSG_MAP(CAudioStat)
	ON_LBN_SELCHANGE(IDC_PART_LIST, OnSelchangePartList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAudioStat message handlers

BOOL CAudioStat::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	return TRUE;  // return TRUE unless you set the focus to a control	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAudioStat::OnOK() 
{
	this->ShowWindow(SW_HIDE);	
	//CDialog::OnOK();
}

void CAudioStat::OnSelchangePartList() 
{
	int pos;

	pos = m_PartsList.GetCurSel();
	if(pos != LB_ERR)
		current = memberPos[pos];
}

void CAudioStat::update(int user, UserData *data)
{
	char text[100];

	check = ++check % 5;
	if(check != 0)
		return;

	if(user == current)
	{
		sprintf(text, "%4.0f kbps", data->getKbps());
		m_Bandwidth.SetSel(0, -1);
		m_Bandwidth.ReplaceSel(text);

		if((data->frames_received + data->frames_lost) > 0)
			sprintf(text, "%6d (%3d%%)", data->frames_received, 100*data->frames_received/(data->frames_received + data->frames_lost));
		else
			sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Received.SetSel(0, -1);
		m_Received.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", data->frames_lost, data->getPercentLost());
		m_Lost.SetSel(0, -1);
		m_Lost.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", data->frames_late, data->getPercentLate());
		m_Late.SetSel(0, -1);
		m_Late.ReplaceSel(text);

		sprintf(text, "%6d", data->frames_received + data->frames_lost);
		m_Total.SetSel(0, -1);
		m_Total.ReplaceSel(text);

		sprintf(text, "%3.0f ms", data->getLatency());
		m_Latency.SetSel(0, -1);
		m_Latency.ReplaceSel(text);
	}
}

void CAudioStat::Vupdate(int user, UserData *data)
{
	char text[100];

	check = ++check % 5;
	if(check != 0)
		return;

	if(user == current)
	{
		sprintf(text, "%4.0f kbps", data->getKbps());
		m_Vbandwidth.SetSel(0, -1);
		m_Vbandwidth.ReplaceSel(text);

		if((data->frames_received + data->frames_lost) > 0)
			sprintf(text, "%6d (%3d%%)", data->frames_received, 100*data->frames_received/(data->frames_received + data->frames_lost));
		else
			sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Vrec.SetSel(0, -1);
		m_Vrec.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", data->frames_lost, data->getPercentLost());
		m_Vlost.SetSel(0, -1);
		m_Vlost.ReplaceSel(text);

		sprintf(text, "%2.1f fps", data->getFps());
		m_Fps.SetSel(0, -1);
		m_Fps.ReplaceSel(text);
	}
}

void CAudioStat::Init()
{
	int i;
	char host[50];
	char text[100];
	int pos;

	current = -1;
	for(i=0; i<MAX_MEMBERS; i++)
		memberPos[i] = -1;

	m_PartsList.ResetContent();
	ZeroFields();

	for(i=0; i<MAX_MEMBERS; i++)
	{
		if((session->memberActive[i]) && (i != session->localMember))
		{
			ncsGetHostName(session->member[i].host, host);
			sprintf(text, "%s@%s", session->member[i].user, host);
			pos = m_PartsList.AddString(text);
			if(pos != LB_ERR)
				memberPos[pos] = i;
		}
	}

	m_PartsList.SetCurSel(0);
	current = memberPos[0];
}

void CAudioStat::ZeroFields()
{
	char text[100];

	sprintf(text, "%4.0f kbps", 0);
		m_Bandwidth.SetSel(0, -1);
		m_Bandwidth.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Received.SetSel(0, -1);
		m_Received.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Lost.SetSel(0, -1);
		m_Lost.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Late.SetSel(0, -1);
		m_Late.ReplaceSel(text);

		sprintf(text, "%6d", 0, 0);
		m_Total.SetSel(0, -1);
		m_Total.ReplaceSel(text);

		sprintf(text, "%3.0f ms", 0);
		m_Latency.SetSel(0, -1);
		m_Latency.ReplaceSel(text);

		sprintf(text, "%4.0f kbps", 0);
		m_Vbandwidth.SetSel(0, -1);
		m_Vbandwidth.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Vrec.SetSel(0, -1);
		m_Vrec.ReplaceSel(text);

		sprintf(text, "%6d (%3d%%)", 0, 0);
		m_Vlost.SetSel(0, -1);
		m_Vlost.ReplaceSel(text);

		sprintf(text, "%2.1f fps", 0);
		m_Fps.SetSel(0, -1);
		m_Fps.ReplaceSel(text);

}
