#include "stdafx.h"
#include "rtp.h"
#include "UserData.h"

UserData::UserData(RTPheader *hdr, int size)
{
	frames_received = 0;
	frames_lost = 0;
	frames_late = 0;
	latency = 0;
	last_ts = GetTickCount();
	last_seq = hdr->seq;
	first_time = GetTickCount();
	bytes = size;
}

int UserData::frame(RTPheader *hdr, int size)
{
	int tmp;
	int ok = 1;

	//if(frames_received == 100)
	//	reset();
	frames_received++;
	bytes += size;
	if(frames_received == 1)
	{
		last_ts = GetTickCount();
		last_seq = hdr->seq;
		first_time = GetTickCount();
	}
	else
	{
		if(hdr->seq < last_seq)
		{
			frames_lost++;
			ok = 0;
		}

		tmp = GetTickCount();
		if(tmp > last_ts + 150)
			frames_late++;
		latency += tmp - last_ts - 100;
		last_ts = tmp;

	}

	return ok;
}

int UserData::getPercentLost()
{
	if(frames_received == 0)
		return 0;

	return frames_lost*100/(frames_received + frames_lost);
}

int UserData::getPercentLate()
{
	if(frames_received == 0)
		return 0;

	return frames_late*100/(frames_received + frames_lost);
}

void UserData::reset()
{
	frames_received = 0;
	frames_lost = 0;
	frames_late = 0;
	bytes = 0;
	latency = 0;
}

double UserData::getFps()
{
	int time;
	double diff;

	time = GetTickCount();
	diff = time - first_time;
	if(diff == 0)
		return 0;
	return (double)1000 * (double)frames_received / diff;
}

double UserData::getKbps()
{
	int time;
	double diff;

	time = GetTickCount();
	diff = time - first_time;
	if(diff == 0)
		return 0;
	return (double)8 * (double)bytes / diff;
}

double UserData::getLatency()
{
	if(frames_received != 0)
		return (double)latency/(double)frames_received;
	
	return 0;
}
