// ConferenceDialog.cpp : implementation file
//

#include "stdafx.h"
#include "DSProxy.h"
#include "ConferenceDialog.h"
#include "ServerLink.h"
#include "ConfParamsDialog.h"

extern CDSProxyApp theApp;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConferenceDialog dialog


CConferenceDialog::CConferenceDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CConferenceDialog::IDD, pParent), m_ServerLink( 0 ), m_SelectedConference( -1 )
{
	//{{AFX_DATA_INIT(CConferenceDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CConferenceDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConferenceDialog)
	DDX_Control(pDX, IDC_PART_LIST, m_ParticipantList);
	DDX_Control(pDX, IDC_CONF_LIST, m_ConferenceList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConferenceDialog, CDialog)
	//{{AFX_MSG_MAP(CConferenceDialog)
	ON_BN_CLICKED(IDCFNEW, OnNewConference)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_CONF_LIST, OnItemchangedConfList)
	ON_BN_CLICKED(IDCFJOIN, OnJoinConf)
	ON_BN_CLICKED(IDCFLEAVE, OnLeaveConf)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConferenceDialog message handlers

BOOL CConferenceDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	if( m_ConferenceList.InsertColumn( 0, "Topic", LVCFMT_LEFT, 270 ) == -1 )
		MessageBox( "(CActiveUsersDialog) column error" );
	
	if( m_ParticipantList.InsertColumn( 0, "Name", LVCFMT_LEFT, 150 ) == -1 )
		MessageBox( "(CActiveUsersDialog) column error" );
	
	if( m_ParticipantList.InsertColumn( 1, "E-mail", LVCFMT_LEFT, 200, 1 ) == -1 )
		MessageBox( "(CActiveUsersDialog) column error" );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CConferenceDialog::OnOK() 
{
	// TODO: Add your message handler code here and/or call default
	ShowWindow( SW_HIDE );	
	GetOwner()->SetFocus();
}

BOOL CConferenceDialog::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// TODO: Add your specialized code here and/or call the base class
	if( ! wParam )
	{
		CServerLink::CDSMessage* msg = ( CServerLink::CDSMessage* )lParam;
		msg->execute();
		delete msg;
		return TRUE;
	}
	else
		return CDialog::OnCommand( wParam, lParam );
}

void CConferenceDialog::serverLink(CServerLink * sl)
{
	m_ServerLink = sl;
}

void CConferenceDialog::OnNewConference() 
{
	// TODO: Add your control notification handler code here
	CConfParamsDialog dlg;
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with OK
		if( ! dlg.m_Topic.IsEmpty() )
			m_ServerLink->new_conference( CServerLink::OPEN, dlg.m_Topic );
	}
	else if (nResponse == IDCANCEL)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}

}

void CConferenceDialog::OnItemchangedConfList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here
	if( pNMListView->uNewState & LVIS_SELECTED )
	{
		m_SelectedConference = pNMListView->iItem;
		UpdateParticipantList( pNMListView->lParam );
	}	
	*pResult = 0;
}

void CConferenceDialog::OnJoinConf() 
{
	// TODO: Add your control notification handler code here
	if( ( m_SelectedConference != -1 )&&( m_ConferenceList.GetSelectedCount() > 0 ) )
	{
		m_ServerLink->join_conference( m_ConferenceList.GetItemData( m_SelectedConference ) );
	}
}

void CConferenceDialog::OnLeaveConf() 
{
	// TODO: Add your control notification handler code here
	if( ( m_SelectedConference != -1 )&&( m_ConferenceList.GetSelectedCount() > 0 ) )
	{
		m_ServerLink->leave_conference( m_ConferenceList.GetItemData( m_SelectedConference ) );
	}
	
}

void CConferenceDialog::UpdateParticipantList( ConferenceKey ckey )
{
	LV_FINDINFO finfo;
	finfo.flags = LVFI_PARAM;
	finfo.lParam = ckey;
	int idx = m_ConferenceList.FindItem( &finfo );
	if( idx == -1 )
	{
		MessageBox( "Cannot find this conference" );
		return;
	}
	if( ( idx != m_SelectedConference )||( m_ConferenceList.GetSelectedCount() == 0 ) )
		return;

	ConferenceEnt& cent = m_Conferences[ckey];
	ConferenceEnt::iterator it = cent.begin();
	if( ! m_ParticipantList.DeleteAllItems() )
	{
		MessageBox( "Cannot clear participant list" );
		return;
	}
	char buf[128];
	LV_ITEM iinfo;
	iinfo.mask = LVIF_TEXT;
	iinfo.pszText = buf;
	iinfo.cchTextMax = sizeof buf;
	while( it != cent.end() )
	{
		finfo.lParam = *it;
		iinfo.iItem = theApp.m_ActiveUsersDialog->m_ActiveUsersList.FindItem( &finfo );
		if( iinfo.iItem != -1 )
		{
			for( iinfo.iSubItem=0; iinfo.iSubItem<2; iinfo.iSubItem++ )
			{
				int row;
				if( ! theApp.m_ActiveUsersDialog->m_ActiveUsersList.GetItem( &iinfo ) )
					MessageBox( "Cannot retrieve info on participant" );
				else
				{
					if( iinfo.iSubItem == 0 )
					{
						if( ( row = m_ParticipantList.InsertItem( 0, iinfo.pszText ) ) == -1 )
							MessageBox( "InsertItem error" );
					}
					else
						if( m_ParticipantList.SetItemText( row, iinfo.iSubItem, iinfo.pszText ) == 0 )
							MessageBox( "SetItemText error" );
				}
			}
		}
		else
			MessageBox( "Participant not found" );
			it++;
	}
}
