#include "stdafx.h"
#include "SAP.h"

#if !defined( _WIN32 )
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int SAP::
open( int port, const char* host )
{
   struct sockaddr_in name;
   if( ( handle_ = socket( PF_INET, SOCK_STREAM, 0 ) ) < 0 )
   {
      perror( "socket" );
      return -1;
   }

   name.sin_family = AF_INET;
   name.sin_port = htons( port );

   struct hostent* hostinfo;
   hostinfo = gethostbyname( host );
   if( ! hostinfo )
   {
#if defined ( _WIN32 )	
	   fprintf( stderr, "%s: error no. %d\n", host, WSAGetLastError() );
#else
	   fprintf( stderr, "%s: %s\n", host, hstrerror( h_errno ) );
#endif
	   return -1;
   }
   name.sin_addr = *( struct in_addr* )hostinfo->h_addr;
   if( connect( handle_, ( struct sockaddr* )&name, sizeof( name ) ) < 0 )
   {
      perror( "connect" );
      return -1;
   }
	return 0;
}

int SAP::
recv_n( void* buffer, size_t size ) const
{
   int st;
   size_t len = size;
   do
   {
      st = ::recv( handle_, ( char* )buffer, len, 0 );
      if( st > 0 )
      {
		 len -= st;
		 buffer = ( char* )buffer + st;
      } 
      else
      {
		 if( st < 0 )
		 {
		    perror( "SAP::recv_n" );
		    return -1;
		 }
		 return 0;
      }
   }
   while( len );
   return size;
}

int SAP::
send_n( void* buffer, size_t size ) const
{
   int st;
   size_t len = size;
   do
   {
      st = ::send( handle_, ( char* )buffer, len, 0 );
      if( st > 0 )
      {
		 len -= st;
		 buffer = ( char* )buffer + st;
      } 
      else
      {
		 if( st < 0 )
		 {
		    perror( "SAP::send_n" );
		    return -1;
		 }
		 return 0;
      }
   }
   while( len );
   return size;
}

int SAP::
recv_n( void* buffer, size_t size, int flags, timeval* tv ) const
{
   int st;
   size_t len = size;
   fd_set fset;
   FD_ZERO( &fset );
   FD_SET( handle_, &fset );
   time_t end_time = time( NULL ) + ( tv ? tv->tv_sec : 0 );
   do
   {
      if( select( handle_ + 1, &fset, NULL, NULL, tv ) < 0 )
      {
		 perror( "SAP::recv_n" );
		 return -1;
      } 
      st = ::recv( handle_, ( char* )buffer, len, flags );
      if( st > 0 )
      {
		 len -= st;
		 buffer = ( char* )buffer + st;
      } 
      else
      {
		 if( st < 0 )
		 {
		    perror( "SAP::recv_n" );
		    return -1;
		 }
		 return 0;
      }
   }
   while( len && time( NULL ) < end_time );
   return size - len;
}

int SAP::
init()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err; 
	wVersionRequested = MAKEWORD( 2, 0 ); 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 )
	{
/* Tell the user that we couldn't find a usable */
/* WinSock DLL.                                 */
		return -1;
	} 
/* Confirm that the WinSock DLL supports 2.0.*/
/* Note that if the DLL supports versions greater    */
/* than 2.0 in addition to 2.0, it will still return */
/* 2.0 in wVersion since that is the version we      */
/* requested.                                        */ 

	if ( LOBYTE( wsaData.wVersion ) != 2 || HIBYTE( wsaData.wVersion ) != 0 )
	{
    /* Tell the user that we couldn't find a usable */
    /* WinSock DLL.                                 */
		WSACleanup();
		return -1;
	}
	/* The WinSock DLL is acceptable. Proceed. */
	return 0;
}