#include "stdafx.h"
#include "ServerLink.h"

#include "msgCREGISTER.h"
#include "msgCUNREG.h"
#include "msgCUSYNCH.h"
#include "msgCINVITE.h"
#include "msgCUSERIP.h"
#include "msgCNEWCONF.h"
#include "msgCCSYNCH.h"
#include "msgCDELCONF.h"
#include "msgCJOIN.h"
#include "msgCLEAVE.h"
#include "msgCINFO.h"

#include <memory>

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int CServerLink::
open( const unsigned short port, const char* host )
{
	if( ! user_dialog_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) invalid user dialog error\n" ), -1 );

	if( ! conf_dialog_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) invalid conf dialog error\n" ), -1 );

	if( msg_pool_.open( DirectoryProtocol::MSGMAX ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msg_pool_ error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::REGISTER, new msgCREGISTER<CDSMsgHeader>( user_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCREGISTER adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::UNREG, new msgCUNREG<CDSMsgHeader>( user_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCUNREG adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::USYNCH, new msgCUSYNCH<CDSMsgHeader>( user_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCUSYNCH adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::INVITE, new msgCINVITE<CDSMsgHeader>( user_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCINVITE adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::USERIP, new msgCUSERIP<CDSMsgHeader>( user_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCUSERIP adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::NEWCONF, new msgCNEWCONF<CDSMsgHeader>( conf_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCNEWCONF adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::CSYNCH, new msgCCSYNCH<CDSMsgHeader>( conf_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCCSYNCH adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::DELCONF, new msgCDELCONF<CDSMsgHeader>( conf_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCDELCONF adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::JOIN, new msgCJOIN<CDSMsgHeader>( conf_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCJOIN adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::LEAVE, new msgCLEAVE<CDSMsgHeader>( conf_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCLEAVE adding error\n" ), -1 );

	if( msg_pool_.add( DirectoryProtocol::INFO, new msgCINFO<CDSMsgHeader>( conf_dialog_ ) ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) msgCLEAVE adding error\n" ), -1 );

	if( proto_.open( &msg_pool_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) Protocol error\n" ), -1 );

	if( sap_.open( port, host ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::open) sap_ open error\n" ), -1 );

	return 0;
}

int CServerLink::
register_user( const char* fn, const char* ln, const char* em )
{
	CDSMessage* msg;
	if( proto_.get( DirectoryProtocol::REGISTER, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::register_user) REGISTER not found\n" ), -1 );
	
	auto_ptr<CDSMessage> auto_msg( msg );
	ActiveUserKey key = 0;
	const char* buf[4] = { ( const char* )&key, fn, ln, em };
	if( msg->set( buf, sizeof buf ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::register_user) message set\n" ), -1 );

	if( msg->send( sap_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::register_user) message send\n" ), -1 );

	return 0;
}

int CServerLink::
join_conference( ConferenceKey conf )
{
	CDSMessage* msg;
	if( proto_.get( DirectoryProtocol::JOIN, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::join_conference) JOIN not found\n" ), -1 );
	
	auto_ptr<CDSMessage> auto_msg( msg );
	ConferenceKey keys[2] = { conf, 0 };
	if( msg->set( &keys, sizeof keys ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::join_conference) message set\n" ), -1 );

	if( msg->send( sap_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::join_conference) message send\n" ), -1 );

	return 0;
}

int CServerLink::
leave_conference( ConferenceKey conf )
{
	CDSMessage* msg;
	if( proto_.get( DirectoryProtocol::LEAVE, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::leave_conference) LEAVE not found\n" ), -1 );
	
	auto_ptr<CDSMessage> auto_msg( msg );
	ConferenceKey keys[2] = { conf, 0 };
	if( msg->set( &keys, sizeof keys ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::leave_conference) message set\n" ), -1 );

	if( msg->send( sap_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::leave_conference) message send\n" ), -1 );

	return 0;
}

int CServerLink::
request_user_synch()
{
	CDSMessage* msg;
	if( proto_.get( DirectoryProtocol::USYNCH, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::register_user) USYNCH not found\n" ), -1 );
	
	auto_ptr<CDSMessage> auto_msg( msg );
	if( msg->send( sap_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::register_user) message send\n" ), -1 );

	return 0;
}

int CServerLink::
request_conf_synch()
{
	CDSMessage* msg;
	if( proto_.get( DirectoryProtocol::CSYNCH, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::request_conf_synch) CSYNCH not found\n" ), -1 );
	
	auto_ptr<CDSMessage> auto_msg( msg );
	if( msg->set( 0 ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::request_conf_synch) message set\n" ), -1 );

	if( msg->send( sap_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::request_conf_synch) message send\n" ), -1 );

	return 0;
}

int CServerLink::
new_conference( ConferenceType ctype, const char* topic )
{
	CDSMessage* msg;
	if( proto_.get( DirectoryProtocol::NEWCONF, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::new_conference) NEWCONF not found\n" ), -1 );
	
	auto_ptr<CDSMessage> auto_msg( msg );
	const char* buf[2] = { ( const char* )&ctype, topic };
	if( msg->set( buf, sizeof buf ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::new_conference) message set\n" ), -1 );

	if( msg->send( sap_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](CServerLink::new_conference) message send\n" ), -1 );

	return 0;
}

int CServerLink::
svc()
{
	CDSMessage* msg;
	for( ; ; )
		if( proto_.recv( sap_, msg ) == -1 )
		{
			AfxMessageBox( "Connection with Directory Server has been reset" );
			return -1;
		}
	return 0;
}

UINT CServerLink::
svc_run( LPVOID pParam )
{
	if( ! pParam )
		return -1;

	CServerLink* slink = ( CServerLink* )pParam;
	return slink->svc();
}
