/*
 * File         : msgCCSYNCH.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 23 11:06:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGCCSYNCH_H)
#define __MSGCCSYNCH_H

#include "ConferenceDialog.h"

template<class HEADER>
class msgCCSYNCH
: public Message<HEADER>
{
public:
	msgCCSYNCH( CConferenceDialog* dlg );
	~msgCCSYNCH();
	int recv( SAP& sap, HEADER& header );
	int send( SAP& sap );
	int set( void* params, size_t len = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgCCSYNCH<HEADER>( ( CConferenceDialog* )user_ ); }

private:
	char* buf_;
};

template<class HEADER>
inline msgCCSYNCH<HEADER>::
msgCCSYNCH( CConferenceDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::CSYNCH ) ), buf_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCCSYNCH<HEADER>::
~msgCCSYNCH() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgCCSYNCH<HEADER>::
recv( SAP& sap, HEADER& header )
{
	delete[] buf_;
	if( !( buf_ = new char[header.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCCSYNCH<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header.msg_length() ) != header.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCCSYNCH<HEADER>::recv) recv error\n" ), -1 );

	header_ = header;
	( ( CConferenceDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCCSYNCH<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCCSYNCH<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCCSYNCH<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCCSYNCH<HEADER>::
set( void* buf, size_t )
{
	delete[] buf_;
	ACE_NEW_RETURN( buf_, char[4], -1 );
	header_.msg_length( 4 );
	return 0;
}

template<class HEADER>
int msgCCSYNCH<HEADER>::
execute()
{
	CConferenceDialog* dlg = ( CConferenceDialog* )user_;
	ConferenceKey ckey = *( ConferenceKey* )( buf_ + sizeof( ConferenceType ) );
	const char* topic = buf_ + sizeof( ConferenceType )
							 + sizeof( ConferenceKey );

	int idx;
	if( ( idx = dlg->m_ConferenceList.InsertItem( 0, topic ) ) == -1 )
	{
		dlg->MessageBox( "(msgCNEWCONF<HEADER>::execute) InsertItem error" );
		return -1;
	}
	if( dlg->m_ConferenceList.SetItemData( idx, ckey ) == 0 )
	{
		dlg->MessageBox( "(msgCNEWCONF<HEADER>::execute) SetItemData error" );
		return -1;
	}

	size_t topic_size = strlen( topic ) + 1;
	ActiveUserKey* pkey = ( ActiveUserKey* )( buf_ + sizeof( ConferenceType )
												   + sizeof( ConferenceKey )
												   + topic_size );

	int numkey = ( header_.msg_length() - sizeof( ConferenceType )
										- sizeof( ConferenceKey )
										- topic_size ) / sizeof( ActiveUserKey );
	
	CConferenceDialog::ConferenceEnt& cent = dlg->m_Conferences[ckey];
	for( int i=0; i<numkey; i++ )
		cent.push_back( pkey[i] );

	return 0;
}

#endif
