/*
 * File         : msgCDELCONF.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 24 11:00:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCDELCONF_H )
#define __MSGCDELCONF_H

#include "ConferenceDialog.h"

template<class HEADER>
class msgCDELCONF
: public Message<HEADER>
{
public:
	msgCDELCONF( CConferenceDialog* dlg );
	~msgCDELCONF();
	int recv( SAP& sap, HEADER& header );
	int send( SAP& sap );
	int set( void* params, size_t len = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgCDELCONF<HEADER>( ( CConferenceDialog* )user_ ); }

protected:
	ConferenceKey key_;
};

template<class HEADER>
inline msgCDELCONF<HEADER>::
msgCDELCONF( CConferenceDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::DELCONF, sizeof key_ ) ), key_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCDELCONF<HEADER>::
~msgCDELCONF() 
{
}

template<class HEADER>
int msgCDELCONF<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCDELCONF<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	( ( CConferenceDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCDELCONF<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCDELCONF<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCDELCONF<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCDELCONF<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCDELCONF<HEADER>::set) setting error\n" ), -1 );

	key_ = *( ConferenceKey* )buf;
	return 0;
}

template<class HEADER>
int msgCDELCONF<HEADER>::
execute()
{
	CConferenceDialog* dlg = ( CConferenceDialog* )user_;
	LV_FINDINFO finfo;
	finfo.flags = LVFI_PARAM;
	finfo.lParam = key_;
	int idx = dlg->m_ConferenceList.FindItem( &finfo );
	if( idx == -1 )
	{
		dlg->MessageBox( "Conference not found" );
		return -1;
	}
	if( dlg->m_ConferenceList.DeleteItem( idx ) == 0 )
	{
		dlg->MessageBox( "Conference cannot be deleted" );
		return -1;
	}
	if( ( idx = dlg->m_ConferenceList.GetNextItem( idx, LVNI_ALL ) ) == -1 )
	{
		if( ! dlg->m_ParticipantList.DeleteAllItems() )
		{
			dlg->MessageBox( "Cannot clear participant list" );
			return -1;
		}
	}
	else
	{
		if( ! dlg->m_ConferenceList.SetItemState( idx, LVIS_SELECTED, LVIS_SELECTED ) )
		{
			dlg->MessageBox( "Cannot set state" );
			return -1;
		}
	}
	CConferenceDialog::ConferenceRep::iterator it = dlg->m_Conferences.find( key_ );
	if( it == dlg->m_Conferences.end() )
	{
		dlg->MessageBox( "Conference info not found" );
		return -1;
	}
	dlg->m_Conferences.erase( it );
	return 0;
}

#endif
