/*
 * File         : msgCINFO.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 26 13:00:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCINFO_H )
#define __MSGCINFO_H

template<class HEADER>
class msgCINFO
: public Message<HEADER>
{
public:
	msgCINFO( CConferenceDialog* dlg );
	~msgCINFO();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgCINFO<HEADER>( ( CConferenceDialog* )user_ ); }

protected:
	char* buf_;
};

template<class HEADER>
inline msgCINFO<HEADER>::
msgCINFO( CConferenceDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::INFO ) ), buf_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCINFO<HEADER>::
~msgCINFO() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgCINFO<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	delete[] buf_;
	if( !( buf_ = new char[header_.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINFO<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINFO<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	( ( CConferenceDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCINFO<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINFO<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINFO<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCINFO<HEADER>::
set( void* buf, size_t len )
{
	delete[] buf_;
	if( !( buf_ = strdup( ( const char* )buf ) ) )
	{
		header_.msg_length( 0 );	
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINFO<HEADER>::set) allocation error\n" ), -1 );
	}
	header_.msg_length( strlen( buf_ ) + 1 );
	return 0;
}

template<class HEADER>
int msgCINFO<HEADER>::
execute()
{
	if( buf_ )
		AfxMessageBox( buf_ );

	return 0;
}

#endif
