/*
 * File         : msgCINVITE.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 10 10:38:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCINVITE_H )
#define __MSGCINVITE_H

#include "ActiveUsersDialog.h"

template<class HEADER>
class msgCINVITE
: public Message<HEADER>
{
public:
	msgCINVITE( CActiveUsersDialog* dlg );
	~msgCINVITE();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgCINVITE<HEADER>( ( CActiveUsersDialog* )user_ ); }

protected:
	ActiveUserKey key_;
};

template<class HEADER>
inline msgCINVITE<HEADER>::
msgCINVITE( CActiveUsersDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::INVITE, sizeof key_ ) ), key_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCINVITE<HEADER>::
~msgCINVITE() 
{
}

template<class HEADER>
int msgCINVITE<HEADER>::
execute()
{
	return 0;
}

template<class HEADER>
int msgCINVITE<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINVITE<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgCINVITE<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINVITE<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINVITE<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCINVITE<HEADER>::
set( void* buf, size_t len )
{
	if( len != 0 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCINVITE<HEADER>::set) setting error\n" ), -1 );

	key_ = *( ActiveUserKey* )buf;
	return 0;
}

#endif
