/*
 * File         : msgCLEAVE.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 26 11:00:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCLEAVE_H )
#define __MSGCLEAVE_H

#include "ConferenceDialog.h"
#include <algorithm>

using namespace std;

template<class HEADER>
class msgCLEAVE
: public Message<HEADER>
{
public:
	msgCLEAVE( CConferenceDialog* dlg );
	~msgCLEAVE();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgCLEAVE<HEADER>( ( CConferenceDialog* )user_ ); }

protected:
	ConferenceKey key_[2];
};

template<class HEADER>
inline msgCLEAVE<HEADER>::
msgCLEAVE( CConferenceDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::LEAVE, sizeof key_ ) )
{
	key_[0] = 0;
	key_[1] = 0;
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCLEAVE<HEADER>::
~msgCLEAVE() 
{
}

template<class HEADER>
int msgCLEAVE<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCLEAVE<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	( ( CConferenceDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCLEAVE<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCLEAVE<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCLEAVE<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCLEAVE<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCLEAVE<HEADER>::set) setting error\n" ), -1 );

	memcpy( key_, buf, sizeof key_ );
	return 0;
}

template<class HEADER>
int msgCLEAVE<HEADER>::
execute()
{
	CConferenceDialog* dlg = ( CConferenceDialog* )user_;
	LV_FINDINFO finfo;
	finfo.flags = LVFI_PARAM;
	finfo.lParam = key_[0];
	int idx = dlg->m_ConferenceList.FindItem( &finfo );
	if( idx == -1 )
	{
		dlg->MessageBox( "Conference not found" );
		return -1;
	}
	CConferenceDialog::ConferenceRep::iterator it = dlg->m_Conferences.find( key_[0] );
	if( it == dlg->m_Conferences.end() )
	{
		dlg->MessageBox( "Conference info not found" );
		return -1;
	}
	CConferenceDialog::ConferenceRep::referent_type::iterator pit
		= find( ( *it ).second.begin(), ( *it ).second.end(), key_[1] );

	if( pit == ( *it ).second.end() )
	{
		dlg->MessageBox( "User info not found" );
		return -1;
	}
	( *it ).second.erase( pit );
	dlg->UpdateParticipantList( key_[0] );
	return 0;
}

#endif
