/*
 * File         : msgCNEWCONF.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 20 18:06:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGCNEWCONF_H)
#define __MSGCNEWCONF_H

#include "ConferenceDialog.h"

template<class HEADER>
class msgCNEWCONF
: public Message<HEADER>
{
public:
	msgCNEWCONF( CConferenceDialog* dlg );
	~msgCNEWCONF();
	int recv( SAP& sap, HEADER& header );
	int send( SAP& sap );
	int set( void* params, size_t len = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgCNEWCONF<HEADER>( ( CConferenceDialog* )user_ ); }

private:
	char* buf_;
};

template<class HEADER>
inline msgCNEWCONF<HEADER>::
msgCNEWCONF( CConferenceDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::NEWCONF ) ), buf_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCNEWCONF<HEADER>::
~msgCNEWCONF() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgCNEWCONF<HEADER>::
recv( SAP& sap, HEADER& header )
{
	delete[] buf_;
	if( !( buf_ = new char[header.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCNEWCONF<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header.msg_length() ) != header.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCNEWCONF<HEADER>::recv) recv error\n" ), -1 );

	header_ = header;
	( ( CConferenceDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCNEWCONF<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCNEWCONF<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCNEWCONF<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCNEWCONF<HEADER>::
set( void* buf, size_t len )
{
	if( len != 2 * sizeof( const char* ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCNEWCONF<HEADER>::set) setting error\n" ), -1 );

	const char** info = ( const char** )buf;
	const size_t info_len0 = sizeof( ConferenceType );
	const size_t info_len1 = sizeof( ConferenceKey );
	const size_t info_len2 = sizeof( ActiveUserKey );
	const size_t info_len3 = strlen( info[1] ) + 1;

	delete[] buf_;
	ACE_NEW_RETURN( buf_, char[info_len0 + info_len1 + info_len2 + info_len3], -1 );
	memcpy( buf_, info[0], info_len0 );
	memcpy( buf_ + info_len0 + info_len1 + info_len2, info[1], info_len3 );
	header_.msg_length( info_len0 + info_len1 + info_len2 + info_len3 );
	return 0;
}

template<class HEADER>
int msgCNEWCONF<HEADER>::
execute()
{
	CConferenceDialog* dlg = ( CConferenceDialog* )user_;
	ConferenceKey ckey = *( ConferenceKey* )( buf_ + sizeof( ConferenceType ) );
	ActiveUserKey ukey = *( ActiveUserKey* )( buf_ + sizeof( ConferenceType )
												   + sizeof( ConferenceKey ) );

	CConferenceDialog::ConferenceEnt& cent = dlg->m_Conferences[ckey];
	cent.push_back( ukey );
	const char* topic = buf_ + sizeof( ConferenceType )
							 + sizeof( ConferenceKey )
							 + sizeof( ActiveUserKey );
	int idx;
	if( ( idx = dlg->m_ConferenceList.InsertItem( 0, topic ) ) == -1 )
	{
		dlg->MessageBox( "(msgCNEWCONF<HEADER>::execute) InsertItem error" );
		return -1;
	}
	if( dlg->m_ConferenceList.SetItemData( idx, ckey ) == 0 )
		dlg->MessageBox( "(msgCNEWCONF<HEADER>::execute) SetItemData error" );

	return 0;
}

#endif
