/*
 * File         : msgCREGISTER.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 29 18:00:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCREGISTER_H )
#define __MSGCREGISTER_H

#include "ActiveUsersDialog.h"

template<class HEADER>
class msgCREGISTER
: public Message<HEADER>
{
public:
	msgCREGISTER( CActiveUsersDialog* dlg );
	~msgCREGISTER();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgCREGISTER<HEADER>( ( CActiveUsersDialog* )user_ ); }

protected:
	char* buf_;
};

template<class HEADER>
inline msgCREGISTER<HEADER>::
msgCREGISTER( CActiveUsersDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::REGISTER ) ), buf_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCREGISTER<HEADER>::
~msgCREGISTER() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgCREGISTER<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing msg 0x%x, user: 0x%x\n", this, user_ ));
	ActiveUserKey key = *( ActiveUserKey* )buf_;
	const char* fn = buf_ + sizeof key;
	const char* ln = fn + strlen( fn ) + 1;
	const char* em = ln + strlen( ln ) + 1;

	CActiveUsersDialog* dlg = ( CActiveUsersDialog* )user_;
	int idx;
	if( ( idx = dlg->m_ActiveUsersList.InsertItem( 0, fn ) ) == -1 )
	{
		dlg->MessageBox( "(msgCREGISTER<HEADER>::execute) InsertItem error" );
		return -1;
	}

	if( dlg->m_ActiveUsersList.SetItemText( idx, 1, em ) == 0 )
		dlg->MessageBox( "(msgCREGISTER<HEADER>::execute) SetItemText error" );

	if( dlg->m_ActiveUsersList.SetItemData( idx, key ) == 0 )
		dlg->MessageBox( "(msgCREGISTER<HEADER>::execute) SetItemData error" );

	return 0;
}

template<class HEADER>
int msgCREGISTER<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	delete[] buf_;
	if( !( buf_ = new char[header_.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCREGISTER<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCREGISTER<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	( ( CActiveUsersDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCREGISTER<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCREGISTER<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCREGISTER<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCREGISTER<HEADER>::
set( void* buf, size_t len )
{
	if( len != 4 * sizeof( const char* ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCREGISTER<HEADER>::set) setting error\n" ), -1 );

	ActiveUserKey key = **( ActiveUserKey** )buf;
	const char** info = ( const char** )buf + 1;
	const size_t info0_len = strlen( info[0] ) + 1;
	const size_t info1_len = strlen( info[1] ) + 1;
	const size_t all_info_len =  sizeof key + 
								 info0_len +
								 info1_len +
								 strlen( info[2] ) +
								 1;
	delete[] buf_;
	ACE_NEW_RETURN( buf_, char[ all_info_len ], -1 );
	memcpy( buf_, &key, sizeof key );
	strcpy( buf_ + sizeof key, info[0] );
	strcpy( buf_ + sizeof key + info0_len, info[1] );
	strcpy( buf_ + sizeof key + info0_len + info1_len, info[2] );
	header_.msg_length( all_info_len );
	return 0;
}

#endif
