/*
 * File         : msgCUNREG.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 29 18:00:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCUNREG_H )
#define __MSGCUNREG_H

#include "ActiveUsersDialog.h"

template<class HEADER>
class msgCUNREG
: public Message<HEADER>
{
public:
	msgCUNREG( CActiveUsersDialog* dlg );
	~msgCUNREG();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgCUNREG<HEADER>( ( CActiveUsersDialog* )user_ ); }

protected:
	ActiveUserKey key_;
};

template<class HEADER>
inline msgCUNREG<HEADER>::
msgCUNREG( CActiveUsersDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::UNREG, sizeof key_ ) ), key_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCUNREG<HEADER>::
~msgCUNREG() 
{
}

template<class HEADER>
int msgCUNREG<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing msg 0x%x, user: 0x%x\n", this, user_ ));
	CActiveUsersDialog* dlg = ( CActiveUsersDialog* )user_;
	LV_FINDINFO finfo;
	finfo.flags = LVFI_PARAM;
	finfo.lParam = key_;
	int idx = dlg->m_ActiveUsersList.FindItem( &finfo );
	if( idx == -1 )
	{
		dlg->MessageBox( "(msgCUNREG<HEADER>::execute) Item not found" );
		return -1;
	}

	if( dlg->m_ActiveUsersList.DeleteItem( idx ) == 0 )
	{
		dlg->MessageBox( "(msgCUNREG<HEADER>::execute) Item cannot be deleted" );
		return -1;
	}

	return 0;
}

template<class HEADER>
int msgCUNREG<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUNREG<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	( ( CActiveUsersDialog* )user_ )->PostMessage( WM_COMMAND, 0, ( LPARAM )this );
	return 0;
}

template<class HEADER>
int msgCUNREG<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUNREG<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUNREG<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCUNREG<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUNREG<HEADER>::set) setting error\n" ), -1 );

	key_ = *( ActiveUserKey* )buf;
	return 0;
}

#endif
