/*
 * File         : msgCUSERIP.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 10 10:38:26 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGCUSERIP_H )
#define __MSGCUSERIP_H

#include "ActiveUsersDialog.h"

template<class HEADER>
class msgCUSERIP
: public Message<HEADER>
{
public:
	msgCUSERIP( CActiveUsersDialog* dlg );
	~msgCUSERIP();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgCUSERIP<HEADER>( ( CActiveUsersDialog* )user_ ); }

protected:
	ActiveUserKey ip_;
};

template<class HEADER>
inline msgCUSERIP<HEADER>::
msgCUSERIP( CActiveUsersDialog* dlg )
: Message<HEADER>( HEADER( DirectoryProtocol::USERIP, sizeof ip_ ) ), ip_( 0 )
{
	user( ( ActiveUser* )dlg );
}

template<class HEADER>
msgCUSERIP<HEADER>::
~msgCUSERIP() 
{
}

template<class HEADER>
int msgCUSERIP<HEADER>::
execute()
{
	return 0;
}

template<class HEADER>
int msgCUSERIP<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &ip_, sizeof ip_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUSERIP<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgCUSERIP<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUSERIP<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &ip_, sizeof ip_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUSERIP<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCUSERIP<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof ip_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCUSERIP<HEADER>::set) setting error\n" ), -1 );

	ip_ = *( u_long* )buf;
	return 0;
}

#endif
