/*
 * File         : msgACCEPT.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 26 11:00:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:38 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGACCEPT_H )
#define __MSGACCEPT_H

#include "DirectoryProtocol.h"
#include "Message.h"

template<class HEADER>
class msgACCEPT
: public Message<HEADER>
{
public:
	msgACCEPT();
	~msgACCEPT();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgACCEPT<HEADER>; }

protected:
	ConferenceKey key_[2];
};

template<class HEADER>
inline msgACCEPT<HEADER>::
msgACCEPT()
: Message<HEADER>( HEADER( DirectoryProtocol::ACCEPT, sizeof key_ ) ),
  key_[0]( 0 ), key_[1]  ( 0 )
{
}

template<class HEADER>
msgACCEPT<HEADER>::
~msgACCEPT() 
{
}

template<class HEADER>
int msgACCEPT<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgACCEPT<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgACCEPT<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::set) setting error\n" ), -1 );

	memcpy( key_, buf sizeof key_ );
	return 0;
}

template<class HEADER>
int msgACCEPT<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing ACCEPT msg 0x%x, user: 0x%x\n",
				this, user_ ));

	Conference* conf;
	DSActiveUserIterator it( *SDSActiveUserRep::instance() );
	it.acquire_read();
	if( SDSActiveUserRep::instance()->find( key_[0], conf ) == -1 )
	{
		it.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::execute) conf %d not found\n", key_[0] ), -1 );
	}
	int result = conf->ACCEPT( user_ );
	it.release();
	DSMessage* msg = 0;
	if( result == -1 )
	{
		//rejected
		if( SDSProtocol::instance()->get( DirectoryProtocol::ERROR, msg ) == -1 )
			ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::execute) protocol error\n" ), -1 );

		if( msg->set( "Your request has been rejected" ) != -1 )
			user_->send( msg );
	}
	else
	{
		//accepted
		if( SDSProtocol::instance()->get( DirectoryProtocol::ACCEPT, msg ) == -1 )
			ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::execute) protocol error\n" ), -1 );

		if( msg->set( &key_, sizeof key_ ) != -1 )
			user_->broadcast( msg );
	}
	delete msg;
	return 0;
}

#endif
