/*********************************************************************
 *
 * File:         control_clbks.c
 *
 * Description:  Messages callbacks
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>

#include "global.h"
#include "session_manage.h"
#include "message_send.h"
#include "control_clbks.h"

/**************************************************************************
 *
 * Callbacks for local control messages
 *
 **************************************************************************/

/* Session messages */

void localClbSessionAddMember(Message *msg, int len)
{
  Participant *part;
  InviteData *data;
  int no;
  int size;

  part = (Participant *)msg->data;
  
  PRINTF("Engine: invitation send to %s@%s\n", part->user, part->host);
  no = sessionGetNoMembers();
  if(no > 0)
    size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    size = sizeof(InviteData);
  data = (InviteData *)malloc(size);
  data->numMembers = no;
  sessionGetMembers(data->parts);
  sendControlMessageToUser(part, reqRemoteSessionInvite, 
			   (char *)data, size);
  free(data);
}

void localClbSessionJoin(Message *msg, int len)
{
  Participant *part;

  part = (Participant *)msg->data;
  sessionActivate();
  sendControlMessageToUser(part, reqRemoteSessionAccept, NULL, 0);  
}

void localClbSessionReject(Message *msg, int len)
{
  Participant *part;

  part = (Participant *)msg->data;
  sessionDeleteMember(part);
  sessionInit();
  sendControlMessageToUser(part, reqRemoteSessionReject, NULL, 0);  
}

void localClbSessionGetMembers(Message *msg, int len)
{
  InviteData *data;
  int no, size;

  no = sessionGetNoMembers();
  if(no > 0)
    size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    size = sizeof(InviteData);
  data = (InviteData *)malloc(size);
  data->numMembers = no;
  sessionGetMembers(data->parts);
  sendControlMessage(LOCAL, -1, indLocalSessionMembers, 
			   (char *)data, size);
  free(data);
}

void localClbSessionQuit(Message *msg, int len)
{
  sendControlMessage(REMOTE, -1, reqRemoteSessionQuit, NULL, 0);
}

void localClbSessionSendToApp(Message *msg, int len)
{
  AppData *data;
  int size;

  data = (AppData *)msg->data;
  size = len - sizeof(Message) + 1;
  sendToAppPort(data->port, data->tag, data->data, size);
  sendControlMessage(REMOTE, -1, reqRemoteSessionSendToApp, msg->data, size);
}

void localClbSessionStartApp(Message *msg, int len)
{
  int size;

  size = len - sizeof(Message) + 1;
  sendControlMessage(REMOTE, -1, reqRemoteSessionStartApp, msg->data, size);
}


/* Application messages */

void localClbAppAttach(Message *msg, int len)
{
  sessionAddApp((SOCKADDR_IN *)msg->data);
}

void localClbAppDettach(Message *msg, int len)
{
  sessionDeleteApp(*(int *)msg->data);
}

/**************************************************************************
 *
 * Callbacks for remote control messages
 *
 **************************************************************************/
void remoteClbSessionInvite(Message *msg, int len)
{
  InviteData *data;
  int no;
  int i;
  int size;

  size = len - sizeof(Message) + 1;
  data = (InviteData *)msg->data;
  if(sessionCreate(&data->parts[0]))
    {
      sessionDesactivate();
      sendControlMessage(LOCAL, 0, indLocalSessionInvited, 
			 (char *)&msg->sender, sizeof(Participant));  
    }
  else
    sendControlMessageToUser(&msg->sender, reqRemoteSessionReject, 
			     NULL, 0);
}


void remoteClbSessionAddRest(Message *msg, int len)
{
  InviteData *data;
  int i, size;
  DWORD no;

  size = len - sizeof(Message) + 1;
  data = (InviteData *)msg->data;
  no = ntohl(data->numMembers);
  for(i=1; i<no; i++)
    sessionAddMember(&data->parts[i]);

  sessionAddMember(&msg->receiver);
  sessionSetLocalMember();
}

void remoteClbSessionAccept(Message *msg, int len)
{
  InviteData *data;
  int size, data_size;
  DWORD no;

  size = len - sizeof(Message) + 1;

  no = sessionGetNoMembers();
  if(no > 0)
    data_size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    data_size = sizeof(InviteData);
  data = (InviteData *)malloc(data_size);
  data->numMembers = htonl(no);
  sessionGetMembers(data->parts);
  sendControlMessageToUser(&msg->sender, reqRemoteSessionAddRest,
			   (char *)data, data_size);
  free(data);

  sendControlMessage(REMOTE, -1, reqRemoteSessionAddMember, 
		     (char *)&msg->sender, sizeof(Participant));

  sessionAddMember(&msg->sender);

  sendControlMessage(LOCAL, 0, indLocalSessionAccept, 
		     msg->data, size);
}

void remoteClbSessionReject(Message *msg, int len)
{
  int size;

  size = len - sizeof(Message) + 1;
  //sessionReject();
  sendControlMessage(LOCAL, 0, indLocalSessionReject, 
		     msg->data, size);  
}


void remoteClbSessionAddMember(Message *msg, int len)
{
  int size;

  size = len - sizeof(Message) + 1;
  sessionAddMember((Participant *)msg->data);
}

void remoteClbSessionQuit(Message *msg, int len)
{
  sessionDeleteMember(&msg->sender);
  //sessionReject();
}  

void remoteClbSessionSendToApp(Message *msg, int len)
{
  AppData *data;
  int size;

  data = (AppData *)msg->data;
  size = len - sizeof(Message) + 1;
  sendToAppPort(data->port, data->tag, data->data, size);
}

void remoteClbSessionStartApp(Message *msg, int len)
{
  int size;

  size = len - sizeof(Message) + 1;
  sendControlMessage(LOCAL, -1, indLocalSessionStartApp, msg->data, size);
}






