#include <MMSYSTEM.H>
#include <winbase.h>

#include "stdafx.h"
#include "Conference.h"
#include "TSaudioProc.h"

/////////////////////////////////
// module variables
/////////////////////////////////

////////////////////////////////////////////////////////////////
// common audio data
WAVEFORMATEX *format;	// format of data
MMRESULT err;			// audio error code
	
////////////////////////////////////////////////////////////////
// audio reading data

// handle for in port
HWAVEIN haudio_in = NULL;	

// reading Thread 
HANDLE hThread; 
DWORD dwThreadId;

// event for synchronize reading
HANDLE hEventIn;

// wave data header
WAVEHDR waveInHeader;	

// reading state
int audioReadActive;
int first = 1;

// conference
Conference *conference;

/////////////////////////////////////////////////////////////////
// audio writing data

// handle for out port
HWAVEOUT haudio_out = NULL;	

// event for synchronize writting(not used)
HANDLE hEventOut;

// wave data header
WAVEHDR waveOutHeader;	

// writing state
int firstOut = 1;

int audioInit(Conference *conf)
{
	char in_event_name[20] = "read";
	char out_event_name[20] = "write";

	conference = conf;
	
	format = (WAVEFORMATEX *)malloc(sizeof(WAVEFORMATEX));

	format->wFormatTag = WAVE_FORMAT_PCM;
	format->nChannels = 1;
	format->nSamplesPerSec = 8000;
    format->nAvgBytesPerSec = 16000; 
    format->nBlockAlign = 2; 
	format->wBitsPerSample = 16;
	format->cbSize = NULL;

	waveInHeader.lpData = new char[PORTION_SIZE];
	waveInHeader.dwBufferLength = PORTION_SIZE;
	waveInHeader.dwFlags = 0L;
	waveInHeader.dwLoops = 0L;

	audioReadActive = 0;

	// create read event
	hEventIn = CreateEvent(NULL, TRUE, TRUE, in_event_name);
	ResetEvent(hEventIn);


	waveOutHeader.lpData = new char[PORTION_SIZE];
	waveOutHeader.dwBufferLength = PORTION_SIZE;
	waveOutHeader.dwFlags = 0L;
	waveOutHeader.dwLoops = 0L;

	// create write event
	hEventOut = CreateEvent(NULL, TRUE, TRUE, out_event_name);

	return 1;
}

void audioClose(void)
{
	free(format);
	CloseHandle(hEventIn);
	CloseHandle(hEventOut);
}


//////////////////////////////////////////////////////////////////
//
// Audio reading functions
//
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// audio reading init
int audioReadInit(void)
{

	audioReadActive = 1;
	first = 1;

    // create reading thread
    hThread = CreateThread( 
        NULL,                        // no security attributes 
        0,                           // use default stack size  
        (LPTHREAD_START_ROUTINE) audioReadThread, // thread function 
        NULL,                // argument to thread function 
        0,                           // use default creation flags 
        &dwThreadId);                // returns the thread identifier 
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
		return 0;
	}

	// opening audio in port
	err = waveInOpen(&haudio_in, WAVE_MAPPER, format, (DWORD)hEventIn, NULL, 
		CALLBACK_EVENT);
	if(err != MMSYSERR_NOERROR)
	{
		MessageBox(NULL, "Can't open audio in port", "Error", MB_OK);
		return 0;
	}
	audioReadChunk();

	return 1;
}

///////////////////////////////////////////////////////////////////////////
// main audio reading thread
DWORD audioReadThread(LPDWORD lpdwParam)
{
	MSG msg;
	char text[100];

	while(1)
	{
		WaitForSingleObject(hEventIn, INFINITE);
		if(audioReadActive)
		{
			if(waveInHeader.dwFlags && WHDR_DONE)
			{
				audioReadChunk();
			}
		}
		else
		{
			MessageBox(NULL, "Thread: audio read close", "Info", MB_OK);
		}

		ResetEvent(hEventIn);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////
// audio reading close function
void audioReadClose(void)
{
	char text[100];
	
	audioReadActive = 0;
	
	TerminateThread(hThread, 0);
	CloseHandle(hThread);
	
	waveInStop(haudio_in);
	err = waveInReset(haudio_in);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't reset audio in port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	waveInUnprepareHeader(haudio_in, &waveInHeader, sizeof(WAVEHDR));
	err = waveInClose(haudio_in);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't close audio in port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	// CloseHandle(haudio_in);
	haudio_in = NULL;
}

///////////////////////////////////////////////////////////////////////////
// Reads audio portions from microphone and saves it to file
void audioReadChunk(void)
{
	DWORD size;
	void *buffer;

	if(!first)
	{	
		waveInUnprepareHeader(haudio_in, &waveInHeader, sizeof(WAVEHDR));
		buffer = malloc(PORTION_SIZE);
		memcpy(buffer, waveInHeader.lpData, PORTION_SIZE);
	}

	if(haudio_in != NULL)
	{
		waveInHeader.dwFlags = 0L;
		waveInHeader.dwLoops = 0L;

		err = waveInPrepareHeader(haudio_in, &waveInHeader, sizeof(WAVEHDR));
		if(err != MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't prepare audio header", "Error", MB_OK);
	
		ResetEvent(hEventIn);

		if(waveInAddBuffer(haudio_in, &waveInHeader, sizeof(WAVEHDR))
				!= MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't add read audio buffer", "Error", MB_OK);

		if(first)
		{
			first = 0;
			waveInStart(haudio_in);
		}
		else
			audioReadHandler(buffer);
	}
}

///////////////////////////////////////////////////////////////////////////
// Handler for read audio chunk
void audioReadHandler(void *buffer)
{
	if(audioReadActive)
	{
		// TODO: place here your read audio handler 
		conference->SendData(buffer);
	}
	
	free(buffer);
}
 
///////////////////////////////////////////////////////////////////////////
// starting handling of read audio buffers
void audioReadStart(void)
{
	audioReadActive = 1;
}

///////////////////////////////////////////////////////////////////////////
// stop handling of read audio buffers
void audioReadStop(void)
{
	audioReadActive = 0;
}


//////////////////////////////////////////////////////////////////
//
// Audio writing functions
//
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// audio writing init
int audioWriteInit(void)
{
	char text[100];
	
	// WaitForSingleObject(hEventOut, INFINITE);
	firstOut = 1;

	if(haudio_out == NULL)
	{
		err = waveOutOpen(&haudio_out, WAVE_MAPPER, format, (DWORD)hEventOut, NULL, 
			CALLBACK_EVENT);
		if(err != MMSYSERR_NOERROR)
		{
			wsprintf(text, "Can't open audio out port %d", err);
			MessageBox(NULL, text, "Error", MB_OK);
			return 0;
		}	

		// waveOutSetVolume(haudio_out, 0x5000);
	}

	return 1;
}

///////////////////////////////////////////////////////////////////////////
// audio writnig close
void audioWriteClose(void)
{
	char text[100];
	
	err = waveOutReset(haudio_out);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't reset audio out port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	waveOutUnprepareHeader(haudio_out, &waveOutHeader, sizeof(WAVEHDR));
	err = waveOutClose(haudio_out);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't close audio out port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	// CloseHandle(haudio_out);
	haudio_out = NULL;
}


///////////////////////////////////////////////////////////////////////////
// setting audio volume
void audioWriteVolume(UINT vol)
{
	waveOutSetVolume(haudio_out, vol);
}

///////////////////////////////////////////////////////////////////////////
// Puts audio portion to speaker
void audioWriteChunk(void *buffer)
{
	//WaitForSingleObject(hEventOut, INFINITE);
	if(!firstOut)
	{
		waveOutUnprepareHeader(haudio_out, &waveOutHeader, sizeof(WAVEHDR));
	}
	else
		firstOut = 0;

	memcpy(waveOutHeader.lpData, buffer, PORTION_SIZE); 
	waveOutHeader.dwFlags = 0L;
	waveOutHeader.dwLoops = 0L;

	err = waveOutPrepareHeader(haudio_out, &waveOutHeader, sizeof(WAVEHDR));
	if(err != MMSYSERR_NOERROR)
		MessageBox(NULL, "Can't prepare audio out header", "Error", MB_OK);
		
	ResetEvent(hEventOut);

	if(waveOutWrite(haudio_out, &waveOutHeader, sizeof(WAVEHDR))
			!= MMSYSERR_NOERROR)
		MessageBox(NULL, "Can't write audio", "Error", MB_OK);
}
