///////////////////////////////////////////////////////////////////////////
// 
// File						: capture.cpp
//
// Description				: capture functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-19-96
//
////////////////////////////////////////////////////////////////////////////
#include <vfw.h>

#include "stdafx.h"
#include "DisplayDlg.h"
#include "h261.h"
#include "h263.h"
#include "capture.h"
#include "conference.h"
#include "VideoProtocol.h"
#include "convert.h"
#include "display.h"

extern HANDLE hCodecEvent;
extern DWORD localH;

DisplayDlg *display;
DisplayDlg *display1;
HWND capWnd;
HDRAWDIB hDraw;
BITMAPINFO *info;
HDC hDc;

int frames11=0;
int totalsize = 0;
int first = 1;

void getInfo(void)
{
	DWORD info_size;
	char text[100];

	info_size = capGetVideoFormatSize(capWnd);
	info = (BITMAPINFO *)GlobalAlloc(GPTR, info_size);
	capGetVideoFormat(capWnd, info, info_size);

	/*
	wsprintf(text, "Bit count: %d", info->bmiHeader.biBitCount);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Width: %d", info->bmiHeader.biWidth);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Height: %d", info->bmiHeader.biHeight);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Compression: %d", info->bmiHeader.biCompression);
	MessageBox(NULL, text, "Info", MB_OK);
	*/

	// BITMAPINFO structure initialization (video format)
	// info->bmiColors = (RGBQUAD *)NULL;
	// info->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	info->bmiHeader.biWidth = 160; //176;
	info->bmiHeader.biHeight = 120; //144;
	// info->bmiHeader.biPlanes = 1;
	info->bmiHeader.biBitCount = 16;
	info->bmiHeader.biCompression = BI_RGB;
	// info->bmiHeader.biSizeImage = 0;
	// info->bmiHeader.biXPelsPerMeter = 0;
	// info->bmiHeader.biYPelsPerMeter = 0;	
	// info->bmiHeader.biClrUsed = 0;
	// info->bmiHeader.biClrImportant = 0;	
	if(!capSetVideoFormat(capWnd, info, sizeof(BITMAPINFO)))
		MessageBox(NULL, "Can't set video format", "Error", MB_OK);
}

void captureInit(HWND parent, int msec)
{
	CAPTUREPARMS params;
	
	// mirror display dialog
	display = new DisplayDlg;

	if(!display->Create(IDD_DISPLAY))
		MessageBox(NULL, "Error creating window", "Error", MB_OK);

	display->SetWindowPos(&display->wndTopMost, 100, 280, 160, 150, 0);

	// capture window
	capWnd = capCreateCaptureWindow(
		(LPSTR) "Mirror",
		WS_CHILD | WS_VISIBLE,
		0, 0, 176, 144,
		display->m_hWnd,
		23);

	// connect to capture driver
	capDriverConnect(capWnd, 0);

	getInfo();
	drawOpen();

	// set callback for frame
	capSetCallbackOnVideoStream(capWnd, captureFrameClb);

	// set setup
	capCaptureGetSetup(capWnd, &params, sizeof(CAPTUREPARMS));
    params.dwRequestMicroSecPerFrame = msec * 1000; 
    params.fMakeUserHitOKToCapture = FALSE; 
    params.wPercentDropForError = 10; 
    params.fYield = TRUE;
    params.wNumVideoRequested = 20; 
    params.fCaptureAudio = FALSE; 
    params.wNumAudioRequested = 1; 
    params.fLimitEnabled = FALSE; 
    params.wTimeLimit = 30; 
	params.fAbortLeftMouse = FALSE; 
    params.fAbortRightMouse = FALSE; 

	capCaptureSetSetup(capWnd, &params, sizeof(CAPTUREPARMS));

	// set overlay preview mode
	// capOverlay(capWnd, TRUE);
	
	//capPreviewScale(capWnd, FALSE);
	//capPreviewRate(capWnd, 100);
	//capPreview(capWnd, TRUE);
}

void captureClose(void)
{
	char text[100];

	//wsprintf(text, "Bytes/frame: %d", totalsize/frames11);
	//MessageBox(NULL, text, "Info", MB_OK);

	display->DestroyWindow();
	delete display;
	capSetCallbackOnFrame(capWnd, NULL);
	capDriverDisconnect(capWnd);
	DestroyWindow(capWnd);
}

void captureGrab(void)
{
	if(!capCaptureSequenceNoFile(capWnd))
		MessageBox(NULL, "Capture error", "Error", MB_OK);
}

LRESULT PASCAL captureFrameClb(HWND hWnd, LPVIDEOHDR lpVHdr)
{
	// MessageBox(NULL, "Frame grabed", "Info", MB_OK);
	captureHandleImage(lpVHdr->lpData);
	return TRUE;
}
 
void drawOpen(void)
{
	CClientDC *cdc;
	PAINTSTRUCT paintst;
	
	cdc = new CClientDC(display);
	cdc->SetMapMode(MM_TEXT);
	hDc = cdc->GetSafeHdc();

	hDraw = DrawDibOpen();
	// info->bmiHeader.biBitCount = 32;
	//if(!DrawDibProfileDisplay(&info->bmiHeader))
	//	MessageBox(NULL, "Error creating draw dib", "Error", MB_OK);
	
	if(!DrawDibBegin(hDraw, hDc, 160, 120, &info->bmiHeader, -1, -1, DDF_HALFTONE ))
		MessageBox(NULL, "Draw Dib begin error", "Error", MB_OK);
}

void draw(void *image)
{
	if(!DrawDibDraw(hDraw, hDc, 0, 0, 160, 120, &info->bmiHeader, image, 0, 0, 160, 120, DDF_SAME_DRAW))// DDF_HALFTONE
		MessageBox(NULL, "Draw Dib draw error", "Error", MB_OK);
}

void captureHandleImage(void *image)
{
	unsigned char *yuvin;

	yuvin = (unsigned char *)GlobalAlloc(GPTR, QCIF_YUV_SIZE);

	draw(image);
	rgb2yuv((WORD *)image, 
			yuvin, &yuvin[QCIF_LUM_SIZE], &yuvin[QCIF_LUM_SIZE + QCIF_CR_SIZE],
			160, 120);

	confCodeImage((char *)yuvin);

	GlobalFree(yuvin);	
}

void captureSetRate(int msec)
{
	CAPTUREPARMS params;
	char text[100];


	capDriverDisconnect(capWnd);
	DestroyWindow(capWnd);
	capWnd = capCreateCaptureWindow(
		(LPSTR) "Mirror",
		WS_CHILD | WS_VISIBLE,
		0, 0, 176, 144,
		display->m_hWnd,
		23);

	// connect to capture driver
	capDriverConnect(capWnd, 0);

	getInfo();

	// set callback for frame
	capSetCallbackOnVideoStream(capWnd, captureFrameClb);

	//capCaptureStop(capWnd);

	capCaptureGetSetup(capWnd, &params, sizeof(CAPTUREPARMS));

	//wsprintf(text, "Get %d, Set %d",params.dwRequestMicroSecPerFrame/1000, msec);
	//MessageBox(NULL, text, "Info", MB_OK);

	params.dwRequestMicroSecPerFrame = msec * 1000; 
    params.fMakeUserHitOKToCapture = FALSE; 
    params.wPercentDropForError = 10; 
    params.fYield = TRUE;
    params.wNumVideoRequested = 20; 
    params.fCaptureAudio = FALSE; 
    params.wNumAudioRequested = 1; 
    params.fLimitEnabled = FALSE; 
    params.wTimeLimit = 30; 
	params.fAbortLeftMouse = FALSE; 
    params.fAbortRightMouse = FALSE; 

	if(!capCaptureSetSetup(capWnd, &params, sizeof(CAPTUREPARMS)))
		MessageBox(NULL, "Capture set setup error", "Error", MB_OK);

	captureGrab();
}