/*********************************************************************
 *
 * File:         common.c
 *
 * Description:  NPAC Conferencing System API, common functions for 
 *               session  and application management 
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <winsock.h>

#include "common.h"

extern ManagerData managerData;
extern FILE *log;
extern FILE *alog;

void nic()
{
}

void nic(void *bu,...)
{
}


int readAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = recv( fd, (char *)buffer, len, 0 );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "readAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}


int writeAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = send( fd, (char *)buffer, len, 0 );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "writeAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}

long gethostid()
{
  char tmp[100];
  struct hostent *hostinfo;
  int id;
  
  gethostname(tmp, 100);
	  
  hostinfo = gethostbyname(tmp);
  if(hostinfo == NULL)
    {
      return -1;
    }

  id = *(long *)hostinfo->h_addr;
  return id;
}


int sendControlMessage(int sock, MsgHeader *header, char *msg, int len)
{
  char buf[MSG_HEADER_LENGTH];
  int length;

  buf[0] = header->tag;
  buf[1] = header->session;
  buf[2] = header->sender.handle;
  strcpy(&buf[3], header->sender.user);
  //header->sender.host = htonl(header->sender.host);
  memcpy(&buf[23], &header->sender.host, sizeof(int));
  buf[27] = header->receiver.handle;
  strcpy(&buf[28], header->receiver.user);
  //header->receiver.host = htonl(header->receiver.host);
  memcpy(&buf[48], &header->receiver.host, sizeof(int));
  length = htonl(len);
  memcpy(&buf[52], &length, sizeof(int));



  if(writeAll(sock, buf, MSG_HEADER_LENGTH) < 0)
    PRINTF(log, "Manager: write error\n");
  if(len > 0)
	if(writeAll(sock, msg, len) < 0)
		PRINTF(log, "Manager: write error\n");

  return 1;
}

int ncsEngineRequest(short tag, char *data, int len)
{
  MsgHeader header;
  DWORD user_len = 20;

  GetUserName(header.sender.user, &user_len);
  header.tag = (unsigned char)tag;
  header.sender.host = gethostid();
  header.dataLength = len;

  if(managerData.socket > 0)
    sendControlMessage(managerData.socket, &header, data, len);

  return 1;
}

int getHostID(char *host)
{
	int id;
	struct hostent *hostinfo;
  
  hostinfo = gethostbyname(host);
  if(hostinfo == NULL)
    {
      return 0;
    }
  
   id = *(long *)hostinfo->h_addr;
  return id;
}

int getHostName(int id, char *host)
{
	int addr;
	struct hostent *hostinfo;
  
	//PRINTF(log, "Getting host name...\n");
	//PRINTF(log, "Host ID %d\n", id);
	addr = id;
	hostinfo = gethostbyaddr((const char FAR *)&addr, 4, PF_INET);
	if(hostinfo == NULL)
	{
      //PRINTF(log, "Host not found\n");
      return 0;
    }
  
	strcpy(host, hostinfo->h_name);
	//PRINTF(log, "Host found: %s\n", hostinfo->h_name);
	return 1;
}

int readMsgHeader(int sock, MsgHeader *header)
{
  char buf[MSG_HEADER_LENGTH];
  int err;

  //PRINTF(log, "Manager: reading header\n");
  err = readAll(sock, buf, MSG_HEADER_LENGTH);
  if(err == 0)
    {
      //PRINTF(log, "Manager: connection broken\n");
      return err;
    }
  if(err < 0)
    {
      //PRINTF(log, "Manager: reading error\n");
      return err;
    }

  header->tag = buf[0];
  header->session = buf[1];
  header->sender.handle = buf[2];
  strcpy(header->sender.user, &buf[3]);
  header->receiver.handle = buf[27];
  strcpy(header->receiver.user, &buf[28]);
  header->sender.host = *(int *)&buf[23];
  header->receiver.host = *(int *)&buf[48];
  //header->sender.host = ntohl(*(int *)&buf[23]);
  //header->receiver.host = ntohl(*(int *)&buf[48]);
  header->dataLength = ntohl(*(int *)&buf[52]);

  return err;
}

/* The lowest level UDP data packages sending procedure */
int sendPacket(int sock, int hostid, short port, char *msg, int len)
{
  struct sockaddr_in addr;
  int n;

  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = hostid;
  addr.sin_port        = port;
  addr.sin_port        = htons(addr.sin_port);

  if((n=sendto(sock, msg, len, 
	 0, (struct sockaddr *)&addr, sizeof(struct sockaddr_in))) < 0)
	 {
		return 0;  
	 }

 return 1;
}


int start_process(char *name)
{
/*
	int pid;

  if((pid = fork()) < 0)
    {
      PRINTF(log, "Manager: fork error\n");
      return -1;
    }
  else if(pid == 0)
    {
      execl(name, 0);
    }
  else
    return pid;
	*/

	return 1;
}

