#include "stdafx.h"

#include <dsound.h>

#include "DSaudio.h"


LPDIRECTSOUND lpDirectSound;
LPDIRECTSOUNDBUFFER DSbuffer;
LPDIRECTSOUNDBUFFER DSprimaryBuffer;

WAVEFORMATEX format;
DSBUFFERDESC dsbdesc;


int DSinit(HWND hwnd) 
{
	if(DS_OK == DirectSoundCreate(NULL, &lpDirectSound,
		NULL)) {
		// Creation succeeded.
		lpDirectSound->SetCooperativeLevel(hwnd, DSSCL_PRIORITY);

		if(!DScreatePrimaryBuf())
			return 0;
		
		if(!DScreateBuffer(&DSbuffer))
			return 0;

		return 1;

	} else {
		// Creation failed.
		return 0;
	}	
	
	return 0;
}

int DScreatePrimaryBuf() 
{
	HRESULT hr;
	
	DSsetFormat(&format);
	
	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
    // Buffer size is determined by sound hardware.
    dsbdesc.dwBufferBytes = 0; 
    dsbdesc.lpwfxFormat = NULL; // Must be NULL for primary buffers.

	hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, &DSprimaryBuffer, NULL);
    if(DS_OK == hr) {
		// Succeeded. Set primary buffer to desired format.
		hr = DSprimaryBuffer->SetFormat(&format);
		if(DS_OK == hr) {
			// If you want to know the buffer size, call GetCaps.
			/*
			dsbcaps.dwSize = sizeof(DSBCAPS);

			(*lplpDsb)->lpVtbl->GetCaps(*lplpDsb, &dsbcaps);
			*lpdwBufferSize = dsbcaps.dwBufferBytes;
			*/
			return 1;
		}
    }

	return 0;

}


void DSsetFormat(WAVEFORMATEX *fmt) 
{
    memset(fmt, 0, sizeof(WAVEFORMATEX));
    fmt->wFormatTag = WAVE_FORMAT_PCM;
	fmt->nChannels = 1;
	fmt->nSamplesPerSec = 8000;
    fmt->nAvgBytesPerSec = 16000; 
    fmt->nBlockAlign = 2; 
	fmt->wBitsPerSample = 16;
}

int DScreateBuffer(LPDIRECTSOUNDBUFFER *lplpDsb)
{
	HRESULT hr;
	
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    // Need default controls (pan, volume, frequency).
    dsbdesc.dwFlags = DSBCAPS_CTRLDEFAULT; 
    // 100-msecond buffer.
    dsbdesc.dwBufferBytes = 10 * format.nAvgBytesPerSec; 
    dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&format;

    // Create buffer.
    hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, lplpDsb, NULL);
    if(DS_OK == hr) {
        // Succeeded. Valid interface is in *lplpDsb.
		(*lplpDsb)->SetVolume(0);
		hr = DSbuffer->Play(0, 0, DSBPLAY_LOOPING);
		if(hr != DS_OK)
			return 0;

		return 1;
    } else {
        // Failed.
        *lplpDsb = NULL;
        return 0;
    }
}

int DSplay(void *buf)
{
    LPVOID lpvPtr1;
    DWORD dwBytes1; 
    DWORD dwBytes2;
    HRESULT hr;

    // Obtain write pointer.
    hr = DSbuffer->Lock(0, 1600, &lpvPtr1, 
        &dwBytes1, NULL, &dwBytes2, DSBLOCK_FROMWRITECURSOR);

    // If DSERR_BUFFERLOST is returned, restore and retry lock.
    if(DSERR_BUFFERLOST == hr) {

        DSbuffer->Restore();
        hr = DSbuffer->Lock(0, 1600, &lpvPtr1, &dwBytes1, NULL, &dwBytes2, DSBLOCK_FROMWRITECURSOR);
    }
    if(DS_OK == hr) {
        // Write to pointers.
        CopyMemory(lpvPtr1, buf, dwBytes1);
        }

	// Release the data back to DirectSound.
    hr = DSbuffer->Unlock(lpvPtr1, dwBytes1, NULL, 0);
    if(DS_OK == hr) {
        // Success.
		return 1;
	}
    // Lock, Unlock, or Restore failed.
    return 0;
}

void DSclose() 
{
	DSbuffer->Stop();
	DSbuffer->Release();

	lpDirectSound->Release();
}



init(LPDIRECTSOUND lpDirectSound, WAVEFORMATEX format);
	play(void *buf);
	close();
