#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <netdb.h>
#include <unistd.h>

#include "event.h"
#include "simulator.h"



int NCSsimulator::init()
{
  char *ncspath;
  char path[100];

  ncspath = getenv("NCS_PATH");
  strcpy(path, ncspath);
  strcat(path, "/recorder.data");
  first = 1;
  
  if((fd = open(path, O_RDONLY)) < 0)
    {
      fprintf(stderr, "Simulator: file open error\n");
      exit(-1);
    }

  if((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      fprintf(stderr, "Simulator: cannot open socket\n");
      return -1;
    }
  
  addr.sin_family = AF_INET;
  addr.sin_port = 0;
  addr.sin_addr.s_addr = gethostid();
  
  /*
  if((bind(sock, &addr, sizeof(struct sockaddr_in))) < 0)
    {
      fprintf(stderr, "Simulator: bind error\n");
      return -1;
    }
    */

  return 1;
}

void NCSsimulator::quit()
{
  close(sock);
  close(fd);
}


void NCSsimulator::run()
{
  int diff;
  struct timeval time, waittime;
  
  while(event.restore(fd) >= 0)
    {
      gettimeofday(&time);
      if(first)
	{
	  starttime.tv_sec = time.tv_sec;
	  starttime.tv_usec = time.tv_usec;
	  starttimestamp.tv_sec = event.timestamp.tv_sec;
	  starttimestamp.tv_usec = event.timestamp.tv_usec;
	  event.send(sock);
	  first = 0;
	}
      else
	{
	  diff = timeDiff(&event.timestamp, &starttimestamp) - timeDiff(&time, &starttime);
	  /*printf("Time difference %d\n", diff);*/
	  if(diff > 0)
	    {
	      waittime.tv_sec = diff/1000;
	      waittime.tv_usec = (diff % 1000) * 1000;
	      select(0, 0, 0, 0, &waittime);
	    }
	  
	  event.send(sock);	  	  
	}
    }
}

/* Difference a-b in miliseconds */
int NCSsimulator::timeDiff(struct timeval *a, struct timeval *b)
{
  int result;

  result = 1000 * (a->tv_sec - b->tv_sec) + (a->tv_usec - b->tv_usec)/1000;

  return result;
}

NCSsimulator simulator;

void quit()
{
  simulator.quit();
  exit(0);
}

void main(void)
{
  signal(SIGINT, quit);
  simulator.init();
  simulator.run();
}



