/*********************************************************************
 *
 * File:         control_clbks.c
 *
 * Description:  Messages callbacks
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"


extern int tmpSocket;
extern int localSocket;
extern int mbrSocket[];
extern int appSocket[];
extern int inviteSocket;

extern int dbState;
int rec_pid;
int sim_pid;

Participant inviting;
int sendInvitation = 0;
int inviteHandle;

/**************************************************************************
 *
 * Callbacks for local control messages
 *
 **************************************************************************/

/* Session messages */

void localClbSessionCreate(MsgHeader *header, char *msg, int len)
{
  struct timeval time;
  int id;

  header->sender.handle = 0;
  cuserid(header->sender.user);
  header->sender.host = gethostid();
  
  gettimeofday(&time);
  id = time.tv_sec % 256;
  setSessionID(id);

  sessionCreate(&header->sender);

  sessionSetLocalMember(0);
}

void localClbSessionAddMember(MsgHeader *header, char *msg, int len)
{
  Participant *part;
  InviteData *data;
  int no;
  int size;

  part = (Participant *)msg;
  
  PRINTF("Engine: invitation send to %s@%d\n", part->user, part->host);
  no = sessionGetNoMembers();
  if(no > 0)
    size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    size = sizeof(InviteData);
  data = malloc(size);
  data->handle = getLocalHandle();
  data->numMembers = no;
  sessionGetMembers(data->parts);

  if(!sendNewMessage(part, reqNewRemoteSessionInvite, 
			   (char *)data, size))
    {
      PRINTF("Engine: cannot connect\n");
      sendLocalMessage(indLocalSessionTimeout, part, sizeof(Participant));
    }
  else
    {
      PRINTF("Engine: invitation received\n");
      sendLocalMessage(indLocalSessionConnected, part, sizeof(Participant));
    }
    

  free(data);
}

void localClbSessionJoin(MsgHeader *header, char *msg, int len)
{
  Participant *part;
  int no;

  part = (Participant *)msg;
  sendInviteMessage(reqRemoteSessionAccept, NULL, 0);  

  mbrSocket[inviteHandle] = inviteSocket;
  inviteSocket = -1;
}

void localClbSessionReject(MsgHeader *header, char *msg, int len)
{
  Participant *part;

  part = (Participant *)msg;
  sessionInit();
  sendInviteMessage(reqRemoteSessionReject, NULL, 0);  
  close(inviteSocket);
  inviteSocket = -1;
}

void localClbSessionGetMembers(MsgHeader *header, char *msg, int len)
{
  InviteData *data;
  int no, size;

  no = sessionGetNoMembers();
  if(no > 0)
    size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    size = sizeof(InviteData);
  data = malloc(size);
  data->numMembers = no;
  sessionGetMembers(data->parts);
  sendLocalMessage(indLocalSessionMembers, 
			   (char *)data, size);
  free(data);
}

void localClbSessionQuit(MsgHeader *header, char *msg, int len)
{
  sendMemberMessage(-1, reqRemoteSessionQuit, NULL, 0);
  sendAppMessage(-1, indLocalAppQuit, NULL, 0);
  quitDaemon();
}

void localClbSessionSendToApp(MsgHeader *header, char *msg, int len)
{
  AppData *data;
  int size, no;

  data = (AppData *)msg;
  no = getAppNumber(data->port);
  PRINTF("App no %d, port %d\n", no, data->port);
  if(no >= 0)
    sendAppMessage(no, data->tag, data->data, len);
}

void localClbSessionStartApp(MsgHeader *header, char *msg, int len)
{
  sendMemberMessage(-1, reqRemoteSessionStartApp, msg, len);
}

void localClbSessionSaveInit(MsgHeader *header, char *msg, int len)
{
  char *ncspath;
  char path[100];

  if(dbState == DbStateIdle)
    {
      ncspath = getenv("NCS_PATH");
      strcpy(path, ncspath);
      strcat(path, "/Database/Recorder/NCSrecorder");
      
      dbState = DbStateSave;
      rec_pid = start_process(path);

      sendAppMessage(-1, indDbStart, NULL, 0);
    }
}

void localClbSessionSaveClose(MsgHeader *header, char *msg, int len)
{
  if(dbState == DbStateSave)
    {
      kill(rec_pid, SIGINT);
      dbState = DbStateIdle;
      
      sendAppMessage(-1, indDbStop, NULL, 0);
    }
}


void localClbSessionRestoreInit(MsgHeader *header, char *msg, int len)
{
  char *ncspath;
  char path[100];

  if(dbState == DbStateIdle)
    {
      ncspath = getenv("NCS_PATH");
      strcpy(path, ncspath);
      strcat(path, "/Database/Simulator/NCSsimulator");
      
      dbState = DbStateRestore;
      sim_pid = start_process(path);
    }
}

void localClbSessionRestoreClose(MsgHeader *header, char *msg, int len)
{
  if(dbState == DbStateRestore)
    {
      kill(sim_pid, SIGINT);  
      dbState = DbStateIdle;
    }
}

void localClbSessionReset(MsgHeader *header, char *msg, int len)
{
  int i;
  MsgHeader hdr;

  sendMemberMessage(-1, reqRemoteSessionQuit, NULL, 0);
  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(getReceiverData(i, &hdr))
	{
	  sessionDeleteMember(i);
	  close(mbrSocket[i]);
	  mbrSocket[i] = -1;
	}
    }
  sendAppMessage(-1, indLocalAppQuit, NULL, 0);
  sessionInit();
}

/**************************************************************************
 *
 * Callbacks for new control messages
 *
 **************************************************************************/
void newClbSessionInit(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len)
{
  sessionInit();
  localSocket = tmpSocket;
  tmpSocket = -1;

  if(sendInvitation)
    {
      sendLocalMessage(indLocalSessionInvited, 
			 (char *)&inviting, sizeof(Participant));  
      sendInvitation = 0;
    }
}

void newClbSessionConnect(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len)
{
  mbrSocket[header->sender.handle] = tmpSocket;
  tmpSocket = -1;
  PRINTF("Member %d connected, socket %d\n", header->sender.handle, mbrSocket[header->sender.handle]);
}

/**/
void newClbAppAttach(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len)
{
  int no;

  no = getFreeAppSocket();
  appSocket[no] = tmpSocket;
  tmpSocket = -1;
  PRINTF("Engine: application port %d, number %d attached\n", ((struct sockaddr_in *)msg)->sin_port, no);
  sessionAddApp(no, (struct sockaddr_in *)msg);
  if(dbState == DbStateSave)
    {
      sendAppMessage(no, indDbStart, NULL, 0);
    }
}

/**/
void newClbSessionInvite(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len)
{
  InviteData *data;
  int no;
  int i;

  data = (InviteData *)msg;
  inviteSocket = tmpSocket;
  inviteHandle = data->handle;
  tmpSocket = -1;
  if(sessionCreated())
    {
      PRINTF("Engine: session exists -- reject\n");
      sendInviteMessage(reqRemoteSessionReject, 
			     NULL, 0);
      close(inviteSocket);
      inviteSocket = -1;

      return;
    }

  if(sessionInitialized())
    {
      PRINTF("Engine: transmit invitation to manager\n");
      sendLocalMessage(indLocalSessionInvited, 
			 (char *)&header->sender, sizeof(Participant));  

      return;
    }

  if(sessionAbandoned())
    {
      PRINTF("Engine: start manager\n");
      memcpy(&inviting, &header->sender, sizeof(Participant));
      sendInvitation = 1;
      startCM();

      return;
    }    
}

/**************************************************************************
 *
 * Callbacks for applications control messages
 *
 **************************************************************************/
/**/
void appClbAppDettach(int no, MsgHeader *header, char *msg, int len)
{
  sessionDeleteApp(no);
  close(appSocket[no]);
  appSocket[no] = -1;
}

void appClbAppData(int no, MsgHeader *header, char *msg, int len)
{
  AppData *data;

  data = (AppData *)msg;
  sendMemberMessage(data->recID, reqRemoteSessionSendAppData, msg, len);
}

/**************************************************************************
 *
 * Callbacks for members control messages
 *
 **************************************************************************/
/**/
void remoteClbSessionAddRest(int no, MsgHeader *header, char *msg, int len)
{
  InviteData *data;
  int i, number, id;

  setSessionID(header->session);

  data = (InviteData *)msg;
  number = data->numMembers;
  sessionCreate(&data->parts[0]);
  sessionSetLocalMember(data->parts[number-1].handle);

  if(no != 0)
    mbrSocket[0] = connect2member(data->parts[0].host);

  for(i=1; i<number; i++)
    {
      if(sessionAddMember(&data->parts[i]))
	{
	  id = data->parts[i].handle;
	  if((id != no) && (i < number - 1))
	    {
	      if(mbrSocket[id] >= 0)
		mbrSocket[getFreeMbrSocket()] = mbrSocket[id];
	      mbrSocket[id] = connect2member(data->parts[i].host);
	    }
	}
    }
}

/**/
void remoteClbSessionAccept(int no, MsgHeader *header, char *msg, int len)
{
  InviteData *data;
  int data_size, number;

  
  header->sender.handle = no;
  sendMemberMessage(-1, reqRemoteSessionAddMember, 
		     (char *)&header->sender, sizeof(Participant));

  sessionAddMember(&header->sender);

  number = sessionGetNoMembers();
  PRINTF("Engine: Send add rest with %d members\n", number);
  if(number > 0)
    data_size = (number - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    data_size = sizeof(InviteData);

  data = malloc(data_size);
  data->numMembers = number;
  sessionGetMembers(data->parts);

  sendLocalMessage(indLocalSessionAccept, 
		     &header->sender, sizeof(Participant));

  sendMemberMessage(no, reqRemoteSessionAddRest,
			   (char *)data, data_size);
  free(data);
}

/**/
void remoteClbSessionReject(int no, MsgHeader *header, char *msg, int len)
{
  sendLocalMessage(indLocalSessionReject, 
		      &header->sender, sizeof(Participant));  
}

/**/
void remoteClbSessionAddMember(int no, MsgHeader *header, char *msg, int len)
{
  sessionAddMember((Participant *)msg);
}

void remoteClbSessionQuit(int no, MsgHeader *header, char *msg, int len)
{
  sessionDeleteMember(no);
  close(mbrSocket[no]);
  mbrSocket[no] = -1;
}  

/**/
void remoteClbSessionSendToApp(int no, MsgHeader *header, char *msg, int len)
{
  AppData *data;
  int number;
  int size;

  data = (AppData *)msg;
  number = getAppNumber(data->port);
  PRINTF("App no %d, port %d\n", no, data->port);
  size = len - sizeof(AppData) + 1;
  if(number >= 0)
    sendAppMessage(number, data->tag, data->data, size);
}

void remoteClbSessionSendAppData(int no, MsgHeader *header, char *msg, int len)
{
  AppData *data;
  int number;

  data = (AppData *)msg;
  number = getAppNumber(data->port);
  PRINTF("App no %d, port %d\n", no, data->port);
  if(number >= 0)
    sendAppMessage(number, indAppData, data, len);
}

/**/
void remoteClbSessionStartApp(int no, MsgHeader *header, char *msg, int len)
{
  sendLocalMessage(indLocalSessionStartApp, msg, len);
}






