/***************************************************************************
 *
 * File name               :browser.c
 * 
 * Description             :NCS conference manager
 *
 * Authors                 :Tomasz Stachowiak, NPAC
 *
 * Date                    :10-23-96
 *
 ***************************************************************************/
#define XP_UNIX

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Form.h>
#include <Xm/DialogS.h>
#include <X11/Shell.h>
#include <Xm/MessageB.h>
#include <Xm/List.h>

/* Motif resource manager (for .uid interface files)*/
#include <Mrm/MrmPublic.h>

#include "tango.h"
#include "launch.c"
#include "../lib/common.h"

typedef struct _ncs_plugin
{
  int active;
  int port;
  char name[50];
  char path[100];
} ncsPlugin;

typedef struct _tangoMsg
{
  unsigned char tag;
  Participant part;
} TangoMsg;

#define MAX_PLUGINS 10

ncsPlugin plugin[MAX_PLUGINS];
int numPlugins = 0;

Participant invitingPart;

int conferenceExists = 0;

char *information;
Arg wArg[10];
int numArg;
XmString label;
int err;

/* 
 * Tango variables 
 */
int tangoAID;
int tangoPort;
int tangoSock;

/* PIDs */
int engine_pid=0;
int audio_pid=0;


Boolean recWorkProc();
void startApp(void *data);

int start_conference(void);
void exit_conference();
void partyAccepted();
void partyRejected();
void partyLeft(Participant *part);
void partyNew(Participant *part);
void SetMessage(const char *msg);
void createMembersInterface();
void createPluginsInterface();

int init_plugins();

void readPlugins();
void readMembers();
void parseInformation(Widget list, char *info);

void mywait(int msec)
{
  struct timeval start, curr;
  int diff=0;

  gettimeofday(&start);
  while(diff < msec)
    {
      gettimeofday(&curr);
      diff = (curr.tv_sec - start.tv_sec)*1000 + (curr.tv_usec - start.tv_usec)/1000;
    }
}

/***********************************************************************
 *
 * Main
 *
 ***********************************************************************/
main( int argc, char **argv)
{
  int port = 12567;

  signal(SIGCHLD, SIG_IGN);

  if(argc != 3)
    {
      fprintf(stderr, "Wrong number of arguments...exiting...\n");
      exit(-1);
    }

  if(start_conference())
    {
      /*sscanf(argv[1], "%d", &port);*/

      sscanf(argv[1], "%d", &tangoAID);
      sscanf(argv[2], "%d", &tangoPort);
      tangoSock = registerApp(tangoAID, tangoPort);

      startApp(&port);
      while(1)
	{
	  recWorkProc();
	}
    }
  else
    {
      /*sscanf(argv[1], "%d", &port);*/
      startApp(&port);
    }
}

/* Invite active users */

void invite_usr(void *data)
{
  struct timeval time;
  Participant *part;

  time.tv_sec = 12;

  mywait(12000);
  /*  select(0, 0, 0, 0, &time);*/

  part = (Participant *)data;
  if(!conferenceExists)
    {
      ncsSessionCreate();
      conferenceExists = 1;
    }


  ncsSessionAddMember(part);
}

/************************************************************************
 *
 * Callbacks regarding plugins management 
 * 
 ************************************************************************/
/*
void info_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  int **pos;
  int count;
  char *text;
  int len;
  char cport[10];

  if(XmListGetSelectedPos(pluginsList, pos, &count))
    {
      len = strlen(plugin[pos[0][0]-1].path) 
	+ strlen(plugin[pos[0][0]-1].name) + 100;
      text = malloc(len);
      strcpy(text, "PLUGIN INFO\nName: ");
      strcat(text, plugin[pos[0][0]-1].name);
      strcat(text, "\nPort: ");
      sprintf(cport, "%d", plugin[pos[0][0]-1].port);
      strcat(text, cport);
      strcat(text, "\nPath: ");
      strcat(text, plugin[pos[0][0]-1].path);

      showMessage(parent, text);

      free(text);
    }
}

void show_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  int **pos;
  int count;

  if(XmListGetSelectedPos(pluginsList, pos, &count))
    {
      if((err = check_process(plugin[pos[0][0]-1].path, 
			      "12hdsa46")) < 0)
	{
	  SetMessage("Can not start plugin");
	  return;
	}
      ncsSessionStartApp(plugin[pos[0][0]-1].port);
    }
}

void hide_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  int **pos;
  int count;

  if(XmListGetSelectedPos(pluginsList, pos, &count))
    {
      if(plugin[i].active)
	{
	  ncsSessionSendToApp(plugin[pos[0][0]-1].port, 
			      indLocalAppQuit, NULL, 0);
	}
    }
}

void accept_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  if(!conferenceExists)
    {
      ncsSessionCreate();
      conferenceExists = 1;
    }
  ncsSessionAcceptInvitation(&invitingPart);
  SetMessage("Invitation to teleconference has been accepted!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
    
  return;
}

void reject_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  ncsSessionRejectInvitation(&invitingPart);
  SetMessage("Invitation to teleconference has been rejected!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
  return;
}
*/


/***************************************************************************
 * 
 * NCS Events callbacks
 * 
 ***************************************************************************/
void startApp(void *data)
{
  int *port;
  int i;

  port = (int *)data;
  for(i=0; i<numPlugins; i++)
    {
      if((plugin[i].port == *port) && (!plugin[i].active))
	{
	  if((audio_pid = start_process(plugin[i].path)) < 0)
	    {
	      return;
	    }
	  plugin[i].active = 1;
	}
    }
}

void partyRejected(Participant *part)
{
}

void partyLeft(Participant *part)
{
}

void partyNew(Participant *part)
{
  /*printf("New member %s@%s\n", part->user, part->host);*/
}

void invited(Participant *part)
{
  /*  printf("I am invited\n");*/
  fflush(stdout);
  if(!conferenceExists)
    {
      ncsSessionCreate();
      conferenceExists = 1;
    }
  memcpy(&invitingPart, part, sizeof(Participant));
  ncsSessionAcceptInvitation(&invitingPart);
}

/***************************************************************************

  Definitions of NCS specific functions

***************************************************************************/  
int start_conference(void)
{ 
  int err;
  char *ncs_path;
  char path[100];
  struct timeval time;

  time.tv_sec = 6;
  time.tv_usec = 0;


  ncs_path = getenv("NCS_TANGO_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/Daemon/NCSengine");

  if ((engine_pid = start_process(path)) < 0)
    {
      /* NCS engine already exists */
    }

  select(0, 0, 0, 0, &time);

  if(ncsSessionInitialize())
    {
      ncsSessionAddCallback(indLocalSessionInvited, invited);
      ncsSessionAddCallback(indLocalSessionAddMember, partyNew);
      ncsSessionAddCallback(indLocalSessionDeleteMember, partyLeft);
      /*      ncsSessionAddCallback(indLocalSessionStartApp, startApp);*/
      
      readPlugins();
    }
  else
    /* session already initialized */
    return 0;

  fflush(stdout);
  fflush(stderr);
  return 1;
}


void exit_conference(void *data)
{
  int i;
  int no=0;
  int *port;
  int audio_port = 12567;

  /* port = (int *)data; */
  
  /*
      if(engine_pid > 0)
	kill(engine_pid, SIGKILL);
      if(audio_pid > 0)
	kill(audio_pid, SIGKILL);
	*/
  port = &audio_port;
  for(i=0; i<numPlugins; i++)
    {
      if(plugin[i].active)
	{
	  if(plugin[i].port == *port)
	    {
	      /*printf("Application %d removed\n",plugin[i].port);*/
	      fflush(stdout);
	      ncsSessionSendToApp(plugin[i].port, indLocalAppQuit, NULL, 0);
	      plugin[i].active = 0;
	    }
	  no++;
	}
    }

  /*  
  if(no < 2)
    {
      if(engine_pid > 0)
	kill(engine_pid, SIGKILL);
      if(audio_pid > 0)
	kill(audio_pid, SIGKILL);
      ncsSessionQuit();
      exit(0);
    }
    */
      ncsSessionQuit();
      exit(0);
}


void readPlugins()
{
  FILE *file;
  char *ncs_path;
  char path[100];

  ncs_path = getenv("NCS_TANGO_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/CM/plugins.ncs");
  file = fopen(path, "r");

  while(!feof(file))
    {
      fscanf(file, "%d\n", &plugin[numPlugins].port); 
      fgets(plugin[numPlugins].name, 50, file);
      strcpy(plugin[numPlugins].path, ncs_path);
      fgets(path, 100, file);
      strcat(plugin[numPlugins].path, path);
      plugin[numPlugins].path[strlen(plugin[numPlugins].path) - 1] = '\0';
      plugin[numPlugins].name[strlen(plugin[numPlugins].name) - 1] = '\0';
      plugin[numPlugins].active = 0;
      numPlugins++;
    }
  fclose(file);
}

/**************************************************************
 * Tango messages handler
 *************************************************************/
void handleTangoMsg(void)
{
  TangoMsg *msg;
  struct Event *event;

  event = receiveEvent(tangoSock);
  msg = (TangoMsg *)getData(event);
  switch(msg->tag)
    {
    case 1:
      /* send invitation */
      /*printf("Tango: invite participant %s@%s\n", msg->part.user, msg->part.host);
      fflush(stdout);*/
      invite_usr(&msg->part);
      break;
      
    case 2:
      /* quit */
      /*printf("Tango: quit plugin\n");
      fflush(stdout);*/
      exit_conference(NULL);
      break;

    default:
      fprintf(stdout, "CM: wrong tango message tag\n");
      fflush(stdout);
    }
  destroyEvent(event);
}

/***************************************************************************
 *
 * Main control messages processing procedure
 *
 ***************************************************************************/
Boolean recWorkProc()
{
  struct timeval timeout;
  fd_set set;
  int sockfd;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;
  sockfd = ncsSessionGetFD(&set);
  FD_SET(tangoSock, &set);
  if(select(FD_SETSIZE, &set, 0, 0, &timeout) > 0)
    {
      /* handle NCS message */
      ncsProcessIncoming(&set);

      /* handle Tango message */
      if(FD_ISSET(tangoSock, &set))
	handleTangoMsg();
    }

  return False;
}










