/*********************************************************************
 *
 * File:         server.c
 *
 * Description:  NPAC Conferencing System API functions for server connection
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         12-17-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "common.h"

#define SERVER_ADDR "mirphak.npac.syr.edu"

struct sockaddr_in serv_addr;
int serv_socket;


void readServerMsg()
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF("Manager: read server message\n");

  err = readMsgHeader(serv_socket, &header);
  if(err == 0)
    {
      PRINTF("Mananger: server connection broken\n");
      return;
    }
  if(err < 0)
    {
      PRINTF("Manager: data reading error\n");
      return;
    }

  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(serv_socket, msg, header.dataLength);
      if(err == 0)
	{
	  PRINTF("Manager: server connection broken\n");
	  return;
	}
      if(err < 0)
	{
	  PRINTF("Manager: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  ncsEngineMessageHandle(header.tag, msg, header.dataLength);
  
  free(msg);
}


int sendServerMessage(int hostid, short tag, char *data, int len)
{
  MsgHeader header;

  if((serv_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
     {
       fprintf(stderr, "Manager: socket opening error\n");
       return -1;
     }

  bzero(&serv_addr, sizeof(struct sockaddr_in));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = hostid; 
  serv_addr.sin_port = htons(SERVER_PORT);

  if(connect(serv_socket, &serv_addr, sizeof(struct sockaddr_in)) < 0)
    {
      PRINTF("Manager: cannot connect to server\n");
      close(serv_socket);
      return -1;
    }

  header.tag = tag;
  cuserid(header.sender.user);
  header.sender.host = gethostid();
  header.dataLength = len;

  sendControlMessage(serv_socket, &header, data, len);

  return 1;
  
}


int serverSendMsg(short tag, char *data, int len)
{
  return sendServerMessage(getHostID(SERVER_ADDR), tag, data, len); 
}

void serverAdd(void)
{
  serverSendMsg(reqServerAddUser, NULL, 0);
  close(serv_socket);
}

void serverDel(void)
{
  serverSendMsg(reqServerDelUser, NULL, 0);
  close(serv_socket);
}


void ncsServerUsers()
{
  if(serverSendMsg(reqServerUsers, NULL, 0))
    readServerMsg();
  close(serv_socket);
}






















































































































