// client.cpp: implementation of the Client class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <winsock.h>

#include "stdafx.h"
#include "client.h"
#include "bva.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int Client::init(int prt, int len)
{
	struct sockaddr_in addr;

	port = prt;
	log = fopen("client.log", "w");
	size = len;
	WSADATA wsaData;	
	running = TRUE;

	if(WSAStartup(257, &wsaData) != 0)
	{
		fprintf(log, "Client: cannot initialize Windows sockets\n");
		return 0;
	}

	// Open main server datagram (UDP) socket
	if ((mainSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
		fprintf(log, "Client: cannot open socket\n");
		return 0;
    }
  
	// Set binding address
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	addr.sin_port        = htons(port);
  
	// Bind socket
	if(bind(mainSocket, (struct sockaddr *)&addr, sizeof(struct sockaddr)) < 0)
    {
		fprintf(log, "Client: cannot bind socket\n");
		return 0;
    }

	serverAddr.sin_family = AF_INET;
	serverAddr.sin_addr.s_addr = getHostID(SERVER_ADDR);
	serverAddr.sin_port        = htons(SERVER_PORT);

	bva = new BVA();
	if(!bva->init())
	{
		fprintf(log, "Client: there is no appropriate audio device\n");
		return 0;
	}

	bvabuffer = bva->createBuffer(1000000);
	if(bvabuffer == NULL)
	{
		fprintf(log, "BV audio buffer init failed\n");
		return 0;
	}
	
	hThread = CreateThread( 
        NULL,                        // no security attributes 
        0,                           // use default stack size  
        (LPTHREAD_START_ROUTINE) runThread, // thread function 
        this,                // argument to thread function 
        0,                           // use default creation flags 
        &dwThreadId);                // returns the thread identifier 
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

	request();

	fprintf(log, "Client started\n");

	return 1;
}

void Client::close()
{
	running = FALSE;
	closesocket(mainSocket);
	bva->close();
	delete bva;
	fclose(log);
}

int Client::request()
{
	int n;
	char msg[100];

	msg[0] = PLAY;
	strcpy(&msg[1], "clip.aud");

	n = sendto(mainSocket, msg, 100, 0, (struct sockaddr *)&serverAddr, sizeof(struct sockaddr_in));
	if(n < 0)
	{
		return 0;  
	}

	return 1;
}

DWORD WINAPI Client::runThread(LPVOID ptr)
{
	struct timeval timeout;
	fd_set set;
	char FAR *data;
	int n;
	int remote_addr_len;
	struct sockaddr_in remote_addr;
	Client *clt = (Client *)ptr;
	int err;
	
	// set timeout
	timeout.tv_sec = 0;
	timeout.tv_usec = 1000;

	remote_addr_len = sizeof(struct sockaddr_in);
	// infinite reading loop
	while(clt->running)
	{
		// check on mainSocket
		FD_ZERO(&set);
		FD_SET(clt->mainSocket, &set);
		
		// received message?
		if(select(clt->mainSocket + 1, &set, 0, 0, &timeout) > 0)
		{
			if(FD_ISSET(clt->mainSocket, &set))
			{
				data = new char[clt->size];
				n = recvfrom(clt->mainSocket, data, clt->size, 0, (struct sockaddr *)&remote_addr, &remote_addr_len); 
      
				if (n < 0)
				{
					err = WSAGetLastError();
					fprintf(clt->log, "Client: recvfrom error: %d\n", err);
				}
				else
					clt->play(data, n);

				delete data;
			}
		}
		
	}

	return 1;
}

void Client::play(char *data, int len)
{
	if(bvabuffer != NULL)
		bvabuffer->play(data, len);	
}

int Client::getHostID(char *host)
{
	int id;
	struct hostent *hostinfo;
  
	hostinfo = gethostbyname(host);
	if(hostinfo == NULL)
    {
		return 0;
    }
	id = *(long *)hostinfo->h_addr;
	
	return id;
}
